.\"{{{  Subroutines and macros
.NH 3
Subroutines and macros
.\"{{{  defmac
.LP
\fB(defmac\fP \fIname\fP \fB(\fP\fIstatement\fP ... \fB))\fP
.SP
A macro (not subroutine!) \fIname\fP will be defined.  Recursive
invocations will be detected and handled inside the macro code.  The
macrocall will be replaced by inline code.
.\"}}}
.\"{{{  deffun
.LP
\fB(deffun\fP \fIname\fP \fB(\fP\fIstatement\fP ... \fB))\fP
.SP
The operations in \fB(\fP\fIstatement\fP ...  \fB)\fP are written to the
\&.origamirc file so that Origami can initialize a fix-macro-buffer.
Recursion is allowed.  In the OCL text following this definition, you
can use \fIname\fP as subroutine call or for key bindings.  You cannot
redefine the fix-macro-buffer.
.\"}}}
.\"{{{  initfun
.LP
\fB(initfun\fP \fIname\fP \fB(\fP\fIstatement\fP ... \fB))\fP
.SP
Like \fBdefmac\fP.  In addition to this, the code is written to the
rc-file to enable the use of \fIname\fP in \fB(keybind\fP \&.\&.\&. \fB)\fP
statements.
.SP
.I Warning:
Using \fBsave-keyboard-macro x\fP has no effect on macros defined with
\fBinitfun\fP when calling them by their name inside a macro.  If you
want to use such an effect you must use \fBexecute-macro x\fP in the
calling macro.
.\"}}}
.\"{{{  usermacro
.LP
\fB(defusermacros\fP \fInumber\fP\fB)\fP
.SP
This command must be used before any \fBdeffun\fP/\fBinitfun\fP
statement.  The macros 1..\fInumber\fP are not used by OCL.  They can be
used for \fBsave-keyboard-macro\fP.
.\"}}}
.\"{{{  which macro places are used
.SP
.I N.B
Using the commands \fBdeffun\fP and \fBinitfun\fP uses the
fix-macro-buffers 1,2,.. (or \fIx\fP,\fIx\fP+1,.., if you have used
\fB(defusermacros\fP \fIx\fP\fB)\fP).
.\"}}}
.\"{{{  no-statusline
.LP
\fB(status-line-off)\fP
.SP
Origami starts without displaying the status line. The Origami commands
\fBstatus-line-on\fP and \fBstatus-line-off\fP can be used during the editing
to override this default.
.\"}}}
.\"{{{  forward
.LP
\fB(forward\fP \fIname\fP\fB)\fP
.SP
\fIname\fP is declared as a name for a macro defined with \fBinitfun\fP
or \fBdeffun\fP.  In the following statements, \fIname\fP can be used as
macro call even before its actual definition with \fB(deffun\fP
\fIname\fP ...  \fB)\fP.
.\"}}}
.\"{{{  automacro
.LP
\fB(read-newfile-macro\fP \fIname\fP\fB)\fP
.SP
The macro named \fIname\fP is called every time a new file is edited,
either explicitly or by entering or exiting a filed fold.
.\"}}}
.\"{{{  abortmacro
.LP
\fB(abort-macro\fP \fIname\fP\fB)\fP
.SP
The macro named \fIname\fP is executed each time the abort key is
pressed.
.\"}}}
.\"{{{  view
.LP
\fB(view-macro\fP \fIname\fP\fB)\fP
.SP
The macro named \fIname\fP is executed at each attempt to change the
file in view mode.
.\"}}}
.\"{{{  promptinmacro
.LP
\fB(begin-prompt-macro\fP \fIname\fP\fB)\fP
.SP
The macro named \fIname\fP is executed each time prompt mode is entered
(except the OCL commands \fBprompt\fP and \fBprompt-counter\fP).  The
repeat argument function reads its arguments in prompt mode.  The macro
may only use the following OCL-elements (some of them are explained in
the following chapters):
.\"{{{  tables of allowed elements
.\"{{{  controls and commands
.ne  8
.TS
tab(|);
l l.
controls:|commands:
\fBif\fP-\fBfi\fP|\fBset-counter\fP
\fBif\fP-\fBelse\fP-\fBfi\fP|\fBset-file-unchanged\fP
\fBwhile\fP|modes,defined in \fB(defmodes\fP \&.\&.\&. \fB)\fP
\fBdo\fP-\fBwhile\fP|user macros using these elements
\fBcase\fP-\fBdefault\fP-\fBesac\fP
\fBlocal\fP
.TE
.\"}}}
.\"{{{  expressions
.ne  10
.TS
tab(|);
l s l
l l l.
booleans:|integer terms:
\fBin-prompt\fP|\fBtest-begin-fold\fP|\fB\+\fP
\fBtest-overwrite\fP|\fBtest-fold-line\fP|\fB\-\fP
\fBtest-view\fP|\fBtest-end-fold\fP|\fBmodulo\fP
\fBtest-autosave\fP|\fBtest-filed\fP|\fBdiv\fP
\fBtest-echo\fP|\fBcounter>0\fP|\fBset-enter\fP
\fBtest-file-changed\fP|\fBcounter-0\fP|\fBset-space-enter\fP
\fBtest-language\fP|\fBlast-message\fP|number constants
\fBtest-text\fP||character constants
.TE
.\"}}}
.\"}}}
.SP
.I Warning:
Keybind doesn't check if you only use the allowed commands! Only
valid commands in such a macro will be executed.  All other
commands will be ignored.
.\"}}}
.\"{{{  promptoutmacro
.LP
\fB(end-prompt-macro\fP \fIname\fP\fB)\fP
.SP
The macro named \fIname\fP is executed each time prompt mode is finished
(except the OCL commands \fBprompt\fP and \fBprompt-counter\fP).  The
command restrictions for the macro are the same as in
\fBbegin-prompt-macro\fP.
.\"}}}
.\"{{{  keynotbound
.LP
\fB(key-not-bound-macro\fP \fIname\fP\fB)\fP
.SP
The macro named \fIname\fP is executed each time an illegal keysequence
was entered.  The command restrictions for the macro are the same as in
\fBbegin-prompt-macro\fP.
.\"}}}
.\"}}}
.\"{{{  control structures
.NH 3
Control structures
.LP
A \fIstatement\fP can be one of the following control structures or
functions:
.\"{{{  while
.LP
\fBwhile\fP \fIcondition\fP \fB(\fP\fIstatement\fP ... \fB)\fP
.SP
Prechecked loop, the macro is repeated while the condition is true.
.\"}}}
.\"{{{  do while
.LP
\fBdo (\fP\fIstatement\fP ... \fB) while\fP \fIcondition\fP
.SP
Postchecked loop, the macro is repeated until the condition is false.
.\"}}}
.\"{{{  if else fi
.LP
\fBif\fP \fIcondition\fP \fB(\fP\fIstatement\fP ... \fB) fi\fP
.br
\fBif\fP \fIcondition\fP \fB(\fP\fIstatement\fP ... \fB) else (\fP\fIstatement\fP ... \fB) fi\fP
.SP
Conditional statement.  If condition is true, the first macro is
executed.  If condition is false, the else macro, if given, is executed.
.\"}}}
.\"{{{  case esac
.LP
\fBcase (\fP\fIcondition\fP \fB(\fP\fIstatement\fP ... \fB))\fP ... \fBesac\fP
.br
\fBcase (\fP\fIcondition\fP \fB(\fP\fIstatement\fP ... \fB))\fP ... \fBdefault (\fP\fIstatement\fP ... \fB) esac\fP
.SP
Conditional statement.  The block of statements following the first
conditional expression evaluating to TRUE is executed.  If none of the
conditions is true, the given default statements are executed.
.\"}}}
.\"{{{  repeat
.LP
\fBrepeat\fP \fIterm\fP \fB(\fP\fIstatement\fP ... \fB)\fP
.SP
\fB(\fP\fIstatement\fP ... \fB)\fP is repeated \fIterm\fP times.
.SP
.I Warning:
Old versions of keybind stored the called statements \fIterm\fP times.
\fIterm\fP had to be a constant!  This new version of keybind generates
code, which uses an internal OCL-variable to count the loops.  The
number of repeats must not be a constant! The repeated statement is
\fBnot\fP called via a subroutine call, so \fBreturn-from-macro\fP will leave
the current macro, not only the \fBrepeat\fP!
.\"}}}
.\"{{{  prompt
.LP
\fBprompt\fP \fIprompt\fP
.SP
Displays \fIprompt\fP below the status line and waits for
\fBnewline-and-indent\fP.  If Origami is in define-macro-mode, the
\fBnewline-and-indent\fP will not be stored! \fIprompt\fP can be:
.\"{{{  decription of prompt
.DS
\fIstring\fP
\fB(\fP\fIprompt-item\fP ... \fB)\fP
.DE
where \fIprompt-item\fP can be:
.DS
\fBcounter\fP \fIname\fP
\fIstring\fP
\fIorigami-message\fP,
.DE
.\"}}}
where \fIorigami-message\fP can be one of the message names declared in
the Origami sources, file msg.  Origami uses empty strings as additional
arguments to fill the format descriptors.  So string will be removed.
Characters and integers will be undefined.
.SP
Origami displays \fIprompt\fP and waits until you type a
\fBnewline-and-indent\fP on your keyboard.  This command doesn't
activate the \fBbegin-\fP/\fBend-prompt-macro\fP's because that could be
done explicitly in the OCL code.  The given input will \fBnot\fP be
interpreted.  Thus, typing a macro name won't execute the macro.
.\"}}}
.\"{{{  exits
.LP
\fBexit\fP
.SP
All running macros are aborted.
.LP
\fBmessage-exit\fP \fIprompt\fP
.SP
Same as exit.  \fIprompt\fP is displayed under the status line.
.\"}}}
.\"{{{  return-from-macro
.LP
\fBreturn-from-macro\fP
.SP
The currently executed macro ends and the caller (keyboard or macro)
resumes execution.
.\"}}}
.\"{{{  local
.LP
\fBlocal (\fP\fIname\fP ... \fB) (\fP\fIstatement\fP ... \fB)\fP
.SP
The values of the variables \fIname\fP ...  are saved during the
execution of \fB(\fP\fIstatement\fP ...  \fB)\fP and restored to their
original values afterwards.
.SP
.I N.B.
\fB(\fP\fIstatement\fP ...  \fB)\fP is executed as a subroutine, so that
using \fBreturn-from-macro\fP inside will only leave this block and not
the current macro.
.\"}}}
.\"{{{  show-cursor
.LP
\fBshow-cursor\fP \fIterm\fP
.SP
The text cursor is displayed and Origami sleeps for \fIterm\fP/10
seconds.  \fIterm\fP is explained in the following chapter
\fBVariables\fP.
.\"}}}
.\"{{{  origami-commands
.LP
\fIorigami-command\fP
.SP
All Origami commands you can use from the keyboard can also be used by
their names.  \fBabort\fP stops the macro execution and calls the abort
macro, if it is defined.
.LP
\fBread-newfile-macro\fP
.br
\fBabort-macro\fP
.br
\fBview-macro\fP
.br
\fBbegin-prompt-macro\fP
.br
\fBend-prompt-macro\fP
.br
\fBkey-not-bound-macro\fP
.SP
If the corresponding hooks are already defined, you can use these names to call
them.
.\"}}}
.\"{{{  strings
.LP
\fIstrings\fP
.SP
Normal text can be used. The syntax is:
.TS
tab(|);
l l.
\fB"\fP\fItext\fP|where text can be a any string not containing:
|<space> \fB"\fP \fB)\fP
\fB"\fP<space>|to use <space>
\fB")\fP|to use \fB)\fP
\fB""\fP|to use \fB"\fP
.TE
.SP
.I Warning:
If you use Origami to edit an OCL file, \fB"\fP<space> should not be the
last statement of a line because Origami deletes all spaces at end of
line.  The problem can be circumvented by appending a comment to the
line.
.\"}}}
.\"}}}
