.\"{{{  variables
.NH 3
Variables
.\"{{{  comments and standard vars
.LP
OCL allows the use of variables.  OCL variables are global integer
variables.  The variable \fBread-tab-width\fP is a always defined.  It
is used to store the tab width for reading files.  Using command line
option \fB\-t\fP\fIn\fP,\fIn\fP>0 initialises it to \fIn\fP, otherwise
it is set to 0.  Using command line option \fB\-t\fP\fIn\fP,\fIn\fP>0
and OCL statements \fBsetcounter read-tab-width\fP
\fIm\fP,\fIm\fP!=\fIn\fP causes Origami to replace the tabs by the
needed number of spaces.
.PP
The three OCL variables \fBscreen-height\fP, \fBscreen-width\fP and
\fBcursor-level\fP are set to size of the text editing area (including
the display area, explained below) on the screen and the number of the
line, the cursor is in (top=1).
.PP
The OCL variable \fBocl-argument\fP can be set by using the command line
option \fB-O\fP\fIvalue\fP, otherwise it is initialised with -1.  This
variable is also used, to store the exit status for \fBshell-command\fP,
\fBshell\fP, \fBpipe-area\fP, \fBfilter-buffer\fP,
\fBpipe-to-command\fP, \fBpipe-from-command\fP  and
\fBpipe-kbd-from-command\fP.
.\"}}}
.\"{{{  defvar
.PP
Except these standard variable, all other OCL variables have to be
declared in the OCL source and they are initialized to zero when Origami
starts up.
.LP
\fB(defvar (\fP\fIvariablename\fP .. \fB))\fP
.SP
All given \fIvariablename\fP are declared as OCL variables.
.\"}}}
.\"{{{  terms
.LP
Variables can be used to build expressions.  The syntax for these
expressions, called \fIterm\fP in this document, is:
.\"{{{  expr
.LP
\fIbasic\fP | [\fB+\fP|\fB\-\fP|\fB*\fP|\fBdiv\fP|\fBmodulo\fP]\fB(\fP\fIterm\fP \fIterm\fP [\fIterm\fP .. ] \fB)\fP
.SP
A \fIterm\fP can be a \fIbasic\fP term or a compound term.
.\"}}}
.LP
OCL's \fIbasic\fP terms are:
.\"{{{  number
.LP
\fInumber\fP
.SP
a signed or unsigned integer constant.
.\"}}}
.\"{{{  char
.LP
\fB"\fP\fIchar\fP
.SP
the integer coding of the given character.
.\"}}}
.\"{{{  name
.LP
\fIname\fP
.SP
a variable
.\"}}}
.\"{{{  pre
.LP
\fBpre (\fP\fIstatement\fP ... \fB)\fP \fIterm\fP
.SP
execute \fB(\fP\fIstatement\fP ...  \fB)\fP and return the value of
\fIterm\fP.
.\"}}}
.\"{{{  bool
.LP
\fBcounter\fP \fIcondition\fP
.SP
is 1, if the boolean expression evaluates to TRUE, otherwise 0.
.\"}}}
.\"{{{  read-repeat
.LP
\fBread-repeat\fP
.SP
Return the value accumulated by \fBset-argument-x\fP.  The repeated
execution of the command will be stopped.
.\"}}}
.\"{{{  store-character
.LP
\fBstore-character\fP
.SP
Return the ASCII value of the character under the cursor.
.\"}}}
.\"{{{  set-enter
.LP
\fBset-enter\fP
.SP
Return number of entered folds.
.\"}}}
.\"{{{  set-space-enter
.LP
\fBset-space-enter\fP
.SP
Return the number of leading spaces hidden by entering folds.
.\"}}}
.\"{{{  store-line
.LP
\fBstore-line\fP
.SP
Return the current line number.
.\"}}}
.\"{{{  store-pos
.LP
\fBstore-pos\fP
.SP
Return the x-position of the cursor.  1 is first displayed character.  This
may be not the first character on that text line if you have entered indented
folds.
.\"}}}
.\"{{{  origami commands
.LP
\fBorigami functions\fP
.SP
All Origami commands can be used in terms.  It's used set variables with
values used in \fBinsert-ascii\fP.
.\"}}}
.LP
\fIop\fP\fB(\fP\fIe1 e2 e3\fP .. \fB)\fP is evaluated like
\fIop\fP\fB(\fP\fIop\fP\fB(\fP\fIe1 e2\fP\fB)\fP \fIe3\fP .. \fB)\fP.
The expressions are not optimized (for example +(8 9) isn't replaced by
17).  Play with the expressions and use viewrc if you need fast/short
macros.
.\"}}}
.LP
The following functions deal with variables/terms and can be used in the
same way like the above mentioned statements:
.\"{{{  prompt-counter
.LP
\fBprompt-counter\fP \fIname prompt\fP
.SP
Prompts \fIprompt\fP below the status line and reads the new value for
\fIname\fP from keyboard.  Input \fIy\fP sets it to 1, \fIn\fP sets it
to 0.  All other inputs have to be decimals or a simple
newline-and-indent.  \fIprompt\fP is explained in section \fBControl
structures\fP.  This command doesn't activate the \fBbegin-\fP /
\fBend-prompt-macro\fP, because you can do it explicitly in the
OCL-code.  The given input will \fBnot\fP be interpreted.  So a given
macro-call won't be executed! When Origami is in define-macro-mode, the
given input isn't stored!
.\"}}}
.\"{{{  prompt-char-counter
.LP
\fBprompt-char-counter\fP \fIname prompt\fP
.SP
Same as \fBprompt-counter\fP, but the counter \fIname\fP is set to the
ASCII-value of the next pressed key.  The value is undefined, if the
pressed key is bound to a macro or Origami-function.  The given input
will \fBnot\fP be interpreted.  So a given macro-call won't be executed!
When Origami is in define-macro-mode, the given input isn't stored!
.\"}}}
.\"{{{  menu-counter
.LP
\fBmenu-counter\fP \fIname prompt\fP
.SP
Display \fIprompt\fP as a menu and ask for the whished item.  Items are
the space separeted textparts in \fIprompt\fP.  You can choose the items
by typing the first character or by moving with \fBbackward-character\fP
and \fBforward-character\fP.  The currently choosen item is displayed
inverse (or with leading and trailling \fB*\fP, if inverse isn't
possible).  The first choosen item is given by \fIname\fP (or 0, if
\fIname\fP is not in the correct range).  The prompt-macros will not be
activated.  \fIname\fP is set to the number of the choosen item (0..).
.\"}}}
.\"{{{  goto-counter
.LP
\fBgoto\fP \fIterm\fP
.SP
Move the cursor on x-position \fIterm\fP.  1 is the first displayed
character.  If you enter a indented fold, position 1 is not the first
character on that line.
.\"}}}
.\"{{{  goto-counter
.LP
\fBgoto-counter\fP \fIterm\fP
.SP
Same as above.
.\"}}}
.\"{{{  goto-line-counter
.LP
\fBgoto-line-counter\fP \fIterm\fP
.SP
Like \fBgoto-line\fP.  The line number is the value of the expression.
.SP
.I Warning:
The line count begins at start-of-file. Origami exits entered
folds and moves to the given position.  Versions before 1.6.17 used
top-of-entered-folds as starting point for counting!
.\"}}}
.\"{{{  counter
.LP
\fBcounter\fP \fIterm\fP
.SP
The value of the expression is returned as ASCII-text.
.\"}}}
.\"{{{  insert-ascii
.LP
\fBinsert-ascii\fP \fIterm\fP
.SP
Insert the character with ASCII-code given by the expression into the input
stream.  There is no check whether the given value of name is a correct
ascii value.  Do it yourself!
.SP
This command can be used too, to put internal OCL-tokens into the token
stream.  This should only used by experienced OCL programmers!  Using
invalid tokens can crash Origami!
.\"}}}
.\"{{{  set-counter
.LP
\fBset-counter\fP \fIname term\fP
.SP
The counter-variable \fIname\fP is set to the value of \fIterm\fP.
.\"}}}
.\"{{{  set-cursor-line
.LP
\fBset-cursor-line\fP \fIterm\fP
.SP
Like \fBredraw-display\fP, but uses line \fIterm\fP instead of the
current line.
.\"}}}
.\"{{{  change-fold-type
.LP
\fBchange-fold-type\fP \fIterm term\fP
.SP
Uses the first \fIterm\fP as foldtype and the second as foldcontents and
changes the attributes of the fold under the cursor.  These values have
specific meanings in TDS!  The values of the \fIterm\fPs are used modulo
255.
.\"}}}
.\"{{{  save-and-exit-code
.LP
\fBsave-and-exit-origami-code\fP \fIname prompt\fP
.SP
Like \fBsave-and-exit-origami\fP, but uses value of OCL-variable
\fIname\fP as exit code.  \fIprompt\fP is displayed as exit-message.
.\"}}}
.\"{{{  old syntax
.LP
The following commands are replaced by corresponding `\fBset-counter\fP
\fIvar term\fP'-statements.  Future releases of Origami are not
guaranteed to support the old forms.
.IP
.ne 11
.TS
tab(|);
l l.
old form:|replace with:
\fBread-repeat\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBread-repeat\fP
\fBstore-char\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBstore-char\fP
\fBset-enter\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBset-enter\fP
\fBset-space-enter\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBset-space-enter\fP
\fBstore-line\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBstore-line\fP
\fBstore-pos\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBstore-pos\fP
\fBstore-char\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fBstore-char\fP
\fBinv-counter\fP \fIx\fP|\fBset-counter\fP \fIx\fP \fB\-(0\fP \fIx\fP\fB)\fP
\fBsum-counter\fP \fIx y\fP|\fBset-counter\fP \fIx\fP \fB\+(\fP\fIx y\fP\fB)\fP
\fBadd-counter\fP \fIx c\fP|\fBset-counter\fP \fIx\fP \fB+(\fP\fIx c\fP\fB)\fP
.TE
.\"}}}
.\"}}}
.\"{{{  character sets
.NH 3
Character sets
.LP
In OCL you can use character sets to simplify boolean expressions.
Predefined character sets are:
.ne 5
.TS
tab(|);
l l.
name:|characters:
\fBalpha\fP|a..zA..Z
\fBlower\fP|a..z
\fBupper\fP|A..Z
\fBdigit\fP|0..9
.TE
You can define your own sets with:
.LP
\fB(defset\fP \fIname\fP \fB(\fP \fIelement\fP ... \fB))\fP
.SP
The character set name is declared.  Elements are all characters given
in the \fIelement\fP-list.  \fIelement\fP can be:
.ne 3
.TS
tab(|);
l l.
\fIname\fP|the name of a known character set
\fB"\fP\fIx\fP|a normal character
\fBC\-\fP\fIx\fP|a control character
.TE
.\"}}}
.\"{{{  boolean expressions
.NH 3
Boolean expressions
.LP
The boolean expressions \fIcondition\fP mentioned above can be:
.\"{{{  boolean basicfunctions
.\"{{{  last-message
.LP
\fBlast-message\fP \fImessage-name\fP
.SP
Is true if the last message created by a string from file msg in the
Origami source directory was \fImessage-name\fP.  A successful test will
set the last displayed message to a dummy value, so that
\fBlast-message\fP \fIx\fP will be false until a new message is
displayed.  Using Origami-messages in prompts changes the last displayed
message.
.\"}}}
.\"{{{  test-file-changed
.LP
\fBtest-file-changed\fP
.SP
Is true if the current file has been changed.
.\"}}}
.\"{{{  test-filed
.LP
\fBtest-filed\fP
.SP
Is true if the current line is a file-folded-line.
.\"}}}
.\"{{{  test-fold-line
.LP
\fBtest-fold-line\fP
.SP
Is true if the current line is a closed-fold-line.
.\"}}}
.\"{{{  test-begin-fold
.LP
\fBtest-begin-fold\fP
.SP
Is true if the current line is the {\&{\&{ line of a open fold.
.\"}}}
.\"{{{  test-end-fold
.LP
\fBtest-end-fold\fP
.SP
Is true if the current line is the }\&}\&} line of a open fold.
.\"}}}
.\"{{{  test-text
.LP
\fBtest-text\fP
.SP
Is true if the current line is a normal text-line.
.\"}}}
.\"{{{  test-top
.LP
\fBtest-top\fP
.SP
Is true if the current line is the top of the entered fold.
.\"}}}
.\"{{{  test-bottom
.LP
\fBtest-bottom\fP
.SP
Is true if the current line is the bottom of the entered fold.
.\"}}}
.\"{{{  test-char
.LP
\fBtest-char "\fP\fIx\fP
.SP
Is true if the character under the cursor is equal to \fIx\fP.
.\"}}}
.\"{{{  test-char-less
.LP
\fBtest-char-less "\fP\fIx\fP
.SP
Is true if the character under the cursor is less then \fIx\fP.
.\"}}}
.\"{{{  test-char-greater
.LP
\fBtest-char-greater "\fP\fIx\fP
.SP
Is true if the character under the cursor is greater than \fIx\fP.
.\"}}}
.\"{{{  test-char-counter
.LP
\fBtest-char-counter\fP \fIterm\fP
.SP
Is true if the ASCII-value of the character under the cursor is equal to
value of the expression.
.\"}}}
.\"{{{  test-char-less-counter
.LP
\fBtest-char-less-counter\fP \fIterm\fP
.SP
Is true if the ASCII-value of the character under the cursor is less
than the value of the expression.
.\"}}}
.\"{{{  test-char-greater-counter
.LP
\fBtest-char-greater-counter\fP \fIterm\fP
.SP
Is true if the ASCII-value of the character under the cursor is greater
than the value of the expression.
.\"}}}
.\"{{{  test-char-set
.LP
\fBtest-char-set\fP \fIname\fP
.SP
Is true if the character under the cursor is in the given character set
\fIname\fP.
.\"}}}
.\"{{{  test-language
.LP
\fBtest-language "\fP\fIx\fP
.SP
Is true if \fIx\fP is the language used by \fBset-language\fP.
.\"}}}
.\"{{{  counter-0
.LP
\fBcounter-0\fP \fIterm\fP
.SP
Is true if the expression evaluates to 0.
.\"}}}
.\"{{{  counter>0
.LP
\fBcounter>0\fP \fIterm\fP
.SP
Is true if the expression evaluates to a value greater than 0.
.\"}}}
.\"{{{  test-begin-line
.LP
\fBtest-begin-line\fP
.SP
Is true if the cursor is placed on or before the first non-space-character
of the current line.
.\"}}}
.\"{{{  test-end-line
.LP
\fBtest-end-line\fP
.SP
Is true if the cursor is placed behind the last non-space-character of the
current line.
.\"}}}
.\"{{{  test-behind-counter
.LP
\fBtest-behind-counter\fP \fIterm\fP
.SP
Is true if the position in line greater than the value of the expression.
.\"}}}
.\"{{{  in-prompt
.LP
\fBin-prompt\fP
.SP
Is true if Origami isn't waiting for a normal edit-character (for
example when prompting for a filename).
.\"}}}
.\"{{{  test-overwrite
.LP
\fBtest-overwrite\fP
.SP
Is true if Origami is in overwrite-mode.
.\"}}}
.\"{{{  test-autosave
.LP
\fBtest-autosave\fP
.SP
Is true if Origami is in autosave-mode.
.\"}}}
.\"{{{  test-echo
.LP
\fBtest-echo\fP
.SP
Is true if Origami is in echo-mode.
.\"}}}
.\"{{{  test-view
.LP
\fBtest-view\fP
.SP
Is true if Origami is in view-mode.
.\"}}}
.\"{{{  term
.LP
\fIterm\fP
.SP
Is true if the given expression evaluates to non-zero.
.\"}}}
.\"}}}
.\"{{{  boolean operators
.\"{{{  pre
.LP
\fBpre (\fP\fIstatement\fP ... \fB)\fP \fIcondition\fP
.SP
The sequence of \fIstatement\fP will be executed before the evaluation of
\fIcondition\fP.
.\"}}}
.\"{{{  not
.LP
\fBnot(\fP\fIcondition\fP\fI)\fP
.SP
The value of \fIcondition\fP is logically negated.
.\"}}}
.\"{{{  and
.LP
\fBand(\fP\fIcondition\fP [ \fIcondition\fP ... ] \fB)\fP
.SP
Is true if all \fIcondition\fPs are true.
.\"}}}
.\"{{{  or
.LP
\fBor(\fP\fIcondition\fP [ \fIcondition\fP ... ] \fB)\fP
.SP
Is true if one of the \fIcondition\fPs is true.
.\"}}}
.\"}}}
.\"{{{  comments
.LP
.I N.B
OCL uses short-circuit evaluation for \fBand\fP and \fBor\fP.  When
Origami is prompting for information (for example, asking to overwrite
an existing file during \fBtoggle-create-file-fold\fP), you can just use
the part of OCL also allowed in begin-prompt- and end-prompt-macros.
For historical reasons it is allowed to use delimiters \fB,\fP in the
expression lists for \fBand\fP and \fBor\fP.
.\"}}}
.\"}}}
