.\"{{{  Notes
.\" This file will be included by origami.ms and is not for seperate use
.\"}}}
.NH 2
OCL syntax and semantic
.\"{{{  general infos
.LP
This section describes all OCL elements.  It is assumed you are
familiar with other programming languages.  You will find examples in a
later section.
.PP
Newlines and comments can be used anywhere in the text between syntactical
structures.  A comment starts with \fB;\fP and ends at end of line.
.PP
Though different machines have different wordlength, we use a 16 bit
coding for internal OCL integers.  Therefore the only machine
dependencies is the character coding (ASCII, ..).
.PP
Internal integers (adresses in macro code and constants in expressions)
are limited to \-32768..32767.
.\"}}}
.\"{{{  filed folds
.NH 3
Handling of filed folds
.LP
Using language \fBNone\fP, keybind will automatically include the contents
of filed folds when reading the source file.  Normally multiple
includes of the same file won't work, because redefinition of macros is not
allowed.  To build a clean macro library, you can use the conditional
compilation explained below.
.\"}}}
.\"{{{  naming the bindings
.NH 3
Name of a binding
.\"{{{  defbinding
.LP
\fB(defbinding\fP \fIname-of-binding\fP\fB)\fP
.SP
Defines a name for the binding which is displayed in the status line of
Origami.  If none is given, \fBOrigami\fP will be displayed.  Multiple
usage of this command is not allowed.  The name is limited to 16
characters.
.\"}}}
.\"}}}
.\"{{{  bindings
.NH 3
Binding commands to keys
.\"{{{  keybind
.LP
\fB(keybind\fP \fIoperation\fP \fIkeysequence\fP\fB)\fP
.SP
The Origami operation named \fIoperation\fP is bound to the keyboard
input \fIkeysequence\fP.  The bindinghelp will contain the entry:
.br
\fB   operation keysequence\fP
.LP
\fB(keybind "\fP\fIsingle-char\fP \fIkeysequence\fP\fB)\fP
.SP
The normal character \fIsingle-char\fP is bound to the keyboard input
\fIkeysequence\fP.  The bindinghelp will contain the entry:
.br
\fB   single-char keysequence\fP.
.br
For example, this form of keybind can be used to map some keys of a
keypad to normal characters like + or * while using others as cursor
keys.
.\"}}}
.\"{{{  alias
.LP
\fB(alias\fP \fIname\fP \fIkeysequence\fP\fB)\fP
.SP
The keyboard input \fIkeysequence\fP can be addressed as
\fB$\fP\fIname\fP in the following definitions.  Using \fB$\fP\fIname\fP
places the symbolic reference name in the bindinghelp rather than the
key sequence itself.
.\"}}}
.\"{{{  description of operands
.\"{{{  name
.LP
\fIname\fP is a normal identifier not beginning with \fB@\fP.
.\"}}}
.\"{{{  keysequence
.LP
A \fIkeysequence\fP is a sequence of \fIkey\fP enclosed in parentheses.
\fIkey\fP specifies a single key (or two keys, because \fBM\-"a\fP requires
two keystrokes).  Valid declarations for \fIkey\fP are:
.\"{{{  table of keys
.ne 11
.TS
tab(|);
l l s
l l s
l l l
l l l
l l s
l l s
l l s
l l l
l l s
.
coding:|meaning:
\fB"\fP\fIx\fP|is the single character \fIx\fP
\fBC\-\fP\fIX\fP|is control-\fIx\fP:|\fB@A\fP..\fBZ[\e]^_\fP codes 0..31
||\fB?\fP codes 127=delete-key
\fBM\-"\fP\fIx\fP|Meta-key (\fBESC\fP) followed by \fIx\fP (case sensitive)
\fBM\-C\-\fP\fIX\fP|META-control-\fIx\fP
\fB$\fP\fIname\fP|the key sequence defined by:
||\fB( alias\fP \fIname\fP \&.\&.\&. \fB)\fP
\fB0x\fP\fIhexdigits\fP|represents the given numer
.TE
.\"}}}
.\"}}}
.\"}}}
.\"{{{  default aliases
.LP
The following alias can be used without declaration:
.DS
\fB$esc\fP = escape key = 27
.DE
Use \fBautoalias\fP to create your alias declarations.
.\"}}}
.\"}}}
.\"{{{  marks
.NH 3
Other marks for folds
.LP
\fB(defmark\fP \fIname\fP \fB(\fP \fImark1\fP \fImark2\fP \fImark3\fP \fImark4\fP \fB))\fP
.SP
This option places the definition of alternative fold marks in the
rcfile, so that Origami can switch to these marks when invoked with
command line option \fB\-m\fP\fIname\fP.  The format for \fImark?\fP is:
.DS
\fB(\fP \fIsingle-char\fP \fIsingle-char\fP \fIsingle-char\fP \fB)\fP
.DE
where \fIsingle-char\fP is a \fB"\fP\fIx\fP definition like in
\fB(keybind\fP ... \fB)\fP.  The meanings of \fImark1\fP ...  \fImark4\fP
are:
.\"{{{  table of marks
.ne 7
.TS
tab(|);
l l l.
item:|used for:|default:
\fImark1\fP|begin of fold|\fB{\&{\&{\fP
\fImark2\fP|closed fold|\fB\&.\&.\&.\fP
\fImark3\fP|name of filed fold|\fB:\&:\&:\fP
|attributes of fold
\fImark4\fP|end of fold|\fB}\&}\&}\fP
.TE
.\"}}}
.\"}}}
.\"{{{  Subroutines and macros/control structures
.\"{{{F toplevel commands and main macro syntax
.\":::F ocl-top.MS
.\"}}}
.so ocl-top.MS
.\"}}}
.\"{{{  vars, bools
.\"{{{F vars and bools in macros
.\":::F ocl-vars.MS
.\"}}}
.so ocl-vars.MS
.\"}}}
.\"{{{  area
.NH 3
The display area
.LP
Origami can use a part of the screen to continuously display a text in
the same manner as the status line.  This area cannot be edited with the
normal editing commands.  The text appears below the status line if it
is enabled, with the last line highlighted.  Otherwise the text appears
below the editing area.  Controlling this text display area is
restricted to the following OCL statements:
.\"{{{  set-area
.LP
\fBset-area\fP \fIterm\fP
.SP
The OCL expression is used to set the size of the new area.  If its value
is 0, no text area will be displayed.  Otherwise the value determines the
number of lines in the new and empty text area.
.\"}}}
.\"{{{  write-area
.LP
\fBwrite-area\fB \fIvariable variable prompt\fP
.SP
Origami uses the variables as x- / y position where to write the
string \fIprompt\fP.
.\"}}}
.\"{{{  pipe-area
.LP
\fBpipe-area\fP \fIprompt\fP
.SP
Origami starts \fIprompt\fP as a command and uses its output as new text
for the text area.  The OCL-variable \fBocl-argument\fP is set to the
exit code from \fIprompt\fP.  Environment variables \fBORIFILE\fP,
\fBORILINE\fP and \fBORIWORD\fP are set to current filename, textline
and word.  Word is the sequence of \fBa\fP..\fBzA\fP..\fBZ_\fP, where
the cursor stands on.  Environment variables \fBORIXPOS\fP and
\fBORIYPOS\fP are set to x position (charater 1,..) and y position (line
1,..) in file.  Environment variable \fBORIPID\fP is set to
\fBOrigami\fP's process id.
.\"}}}
.\"}}}
.\"{{{  modes
.NH 3
User defined modes
.LP
In addition to the predefined modes you can define up to 32 new pairs of
strings which can be displayed as modes in the status line:
.LP
\fB(defmodestring\fP \fIname longstring shortstring\fP\fB)\fP
.SP
\fPname\fP is a new mode.  \fIlongstring\fP is displayed in the status line
when the mode is active.  If the status line becomes to long,
\fIshortstring\fP is used.  \fIlongstring\fP and \fIshortstring\fP are normal
strings.  \fIname\fP is a normal identifier.
.LP
Usermodes are controlled by the following Origami commands.  They can be
used as normal Origami commands in every \fB(deffun \fP\&.\&.\&.\fB )\fP,
\fB(initfun \fP\&.\&.\&.\fB )\fP and \fB(defmac \fP\&.\&.\&.\fB )\fP.
.\"{{{  set-user-mode
.LP
\fBset-user-mode\fP \fIname\fP
.SP
Usermode \fIname\fP is activated.  At this point, \fIname\fP must be
defined as usermode!
.\"}}}
.\"{{{  reset-user-mode
.LP
\fBreset-user-mode\fP \fIname\fP
.SP
Usermode \fIname\fP is reset.  At this point, \fIname\fP must be defined
as usermode!
.\"}}}
.\"}}}
.\"{{{  multiple keyboards
.NH 3
Multiple keyboard modes
.\"{{{  declaration
.LP
To create more complex keybindings (for example a \fBvi\fP emulation with
modes), OCL offers the possibility to use up to eight different
keyboard translation tables.  To do this, you must declare the used
key tables with:
.LP
\fB(defmodes\fP \fIname\fP ... \fB)\fP
.SP
where \fIname\fP is a normal string.  This command has two effects:
.IP \0\0\0\0\(bu 1.5c
Keybind initializes a key table for each \fIname\fP.
.IP \0\0\0\0\(bu 1.5c
The macros \fIname\fP are defined and can be used within other
macro definitions to switch to a certain keytab.  They are not stored in
fixed macro buffers, but you can use them for your one macro definitions.
.\"}}}
.\"{{{  bindings
.LP
The first defined mode is active when starting Origami.  The definition
of a declared keyboard can be done with:
.LP
\fB(mode\fP \fIname\fP \fB(keybind\fP ... \fB)\fP ... \fB)\fP
.SP
All given bindings are put into the keytable for <name>.
\fB(keybind abort ...  )\fP must be declared outside any
\fB(mode\fP ... \fB)\fP statement, which means you can
only have one abort key for all modes.  If using multiple keyboard
modes, no other \fB(keybind\fP ... \fB)\fP  statement is allowed outside
\fB(mode\fP ... \fB)\fP statements.
.br
Using this statement more than once for a specific keytable adds all
given bindings to the given mode.
.\"}}}
.\"}}}
.\"{{{  terminal
.NH 3
Terminals
.LP
You can configure Origami for different terminals in one rc file by using
\fBterminal\fP.  Terminal dependent aliases can be declared on toplevel
via:
.\"{{{  terminal alias
.LP
\fB(terminal\fP \fIname\fP \fB(alias\fP ... \fB)\fP ... \fB)\fP
.br
\fB(terminal (\fP\fIname\fP ... \fB) (alias\fP ... \fB)\fP ... \fB)\fP
.SP
The corresponding alias definitions will be used for the different
terminals.  Terminal dependent aliases inside the \fB(alias\fP ...
\fB)\fP statements are handled for each terminal.  The given aliases are
internally stored as \fItermname\fP\fB@\fP\fIaliasname\fP.  Use \fBautoalias\fP
to genereate your alias declaration.
.\"}}}
.LP
Terminal dependent bindings can be used inside and outside of
\fB(mode\fP \&.\&.\&.  \fB)\fP statements.  The declaration has to be
made like:
.\"{{{  terminal keybind
.LP
\fB(terminal\fP \fIname\fP \fB(keybind\fP ... \fB)\fP ... \fB)\fP
.br
\fB(terminal (\fP \fIname\fP ... \fB) (keybind\fP ... \fB)\fP ... \fB)\fP
.SP
The \fB(keybind\fP \&..\&.\&\fB )\fP commands inside this statement are
only active if Origami is used on a terminal named \fIname\fP (value of
environment variable ORITERM or TERM).  Used aliases have to be declared
globally or must be declared as terminal dependent for all named
terminals.  If a alias isn't declared for one of the given terminals,
the bindings will not be done for this terminal and a errormessage will
be printed.
.\"}}}
.LP
Some different terminals use the same character sequences for their
different keys. To prevent multiple keytables for these terminals, you
can use:
.\"{{{  termalias
.LP
\fB(termalias\fP \fItermname\fP \fB(\fP \fIaliastermname\fP ... \fB))\fP
.SP
If \fB$TERM\fP/\fB$ORITERM\fP is set to one of the \fIaliastermname\fPs,
the keytable for \fItermname\fP is used.
.\"}}}
.\"}}}
.\"{{{  conditional compilation
.NH 3
Conditional compilation
.LP
OCL offers the possibility to use the same OCL source for different
keybindings by using the commands \fB@if-using\fP and \fB@use\fP
to control the compilation of the OCL source.  Some parts of the
source will be handled like comments (like #if .. #endif in C).
.\"{{{  use (..)
.LP
\fB@use (\fP \fIname\fP ... \fB)\fP
.SP
From now on the identifiers \fIname\fP ... are known as set tags.
.\"}}}
.\"{{{  use not(..)
.LP
\fB@use not(\fP \fIname\fP ... \fB)\fP
.SP
From now on the identifiers \fIname\fP ...  are not known as set tags.
Deleting tags automatically created by \fBalias\fP has no influence on
the alias definition.
.\"}}}
.\"{{{  if-using (..)
.LP
\fB@if-using (\fP \fItag\fP ... \fB)\fP \fIstatement\fP ...  \fB@fi\fP
.SP
The list of statements will only be compiled if one of the tags is set.
.\"}}}
.\"{{{  if-using not(..)
.LP
\fB@if-using not(\fP \fItag\fP ... \fB)\fP  \fIstatement\fP ...  \fB@fi\fP
.SP
The list of statements will only be compiled if none of the tags is set.
.\"}}}
.LP
.I Warning:
If a filed fold begins appears inside \fB@if-using
(\fP\fInot-defined-tag\fP\fB)\fP, the fold will not be read to speed up
compilation.  Thus, its impossible to start a \fB@if-using\fP-block in
one file and end it in another !
.LP
All known aliases are known as set tags too.  This can be used to write
OCL sources which adapt to the terminal environment automatically.
Alias-tags can be removed using \fB@use not(\fP\fI$aliasname\fP\fB)\fP
without influencing the alias definition.
.\"}}}
.\"{{{  specials
.NH 3
Special commands
.LP
The following functions are special commands, for internal use. Only
experienced OCL programmers should use these functions.
.\"{{{  screen-off
.LP
\fBscreen-off\fP
.SP
Turn off the screen display.  If keyboard input is expected and the screen is
switched off, the abort key is automatically invoked and the display turned
on again.
.\"}}}
.\"{{{  screen-on
.LP
\fBscreen-on\fP
.SP
Turn on the screen display.  You have to call \fBredraw-display\fP yourself.
.\"}}}
.\"{{{  K_SHIFT
.LP
\fBshift-next-token\fP
.SP
You can use it to control Origami from a pipe regardless of the key
bindings except \fBshift-next-token\fP and \fBquote-character\fP.
There are no other useful applications for it!
.\"}}}
.\"}}}
.\"{{{  references
.NH 3
References
.LP
The following commands can be used to generate additional text files
while compiling new bindings.
.\"{{{  reference-begin/end
.LP
\fB@reference-begin\fP
.br
<comments>
.br
:
.br
\fB@reference-end\fP
.SP
If the command line option \fB\-r\fP\fIreferencefile\fP is active, all
comments will be put into \fIreferencefile\fP.  The text between the
\fB@reference-begin\fP / \fB-end\fP marks will not be parsed as OCL
code, so that the commands must appear at the beginning of a line and
must be followed by a newline.  If the command line option \fB\-R\fP is
active, the data will copied to the rc file.  Origami can display it
with \fBhelp\fP, if no other helpfile is given.
.\"}}}
.\"}}}
