/* This file is part of the origami distrubution. (Amiga Port)
 *
 * Source code written by Thomas Hadig in September 1991
 * Version : 1.6.39 gamma, April 1992
 * (C) 1992 by Thomas Hadig
 * may be distributed unchanged ! (see copyright notes)
 */

/*{{{  compiler defines (switches for features)*/
/* There are some switches, that enable several features in origami :
 * ASL : will use the asl requester, if asl.library is available
 * ARP : will use the arp requester, if arp.library is available
 * PRIORITY : will change origamis priority, when activated (3)
 * AREXX : will enable the arexx port (isn't functional) (only with SAS)
 * MOUSY : will enable mouse using
 * MENU  : will enable menus
 * PAL :   will change default screen size to PAL (256 lines)
 * VIRTUAL : will enable virtual file storage
 * ICONIFY : will enable the inconify feature
 * OS20 / OS13 :
 *     only OS13 : will run on all kickstart version
 *     only OS20 : will only run on kickstart version 2.0 and higher
 *     OS13 and OS20 : will run on all kickstart version, but will
 *                     respect some sepecial 2.0 features.
 *     none : will define OS13 and OS20
 *
 * There are some switches that define some of the above features :
 * REQUEST : ARP & ASL
 * NORMAL : MENU & MOUSY
 * BIG1 : REQUEST & PRIORITY & NORMAL
 * BIG2 : OS20 & OS13
 *
 * BIG will always enable all functions !
 */

#define HOMERC
#define ORIGAMIPATH "S:"
#define NOSYSEXIT

#ifdef BIG
#  define BIG2
#  define BIG1
#  define ICONIFY
#endif
#ifdef BIG1
#  define REQUEST
#  define PRIORITY
#  define NORMAL
#endif
#ifdef BIG2
#  define OS13
#  define OS20
#endif
#ifdef REQUEST
#  define ASL
#  define ARP
#endif
#ifdef NORMAL
#  define MENU
#  define MOUSY
#endif
#if (!(defined(OS20)|(defined(OS13))))
#  define OS13
#  define OS20
#endif
#if (defined(OS20)&defined(OS13))
#  define OSall
#endif
/*}}}  */

/*{{{  additional includes*/
#include "exec/types.h"
#include <stdio.h>
/*}}}  */
/*{{{  stdio-stuff*/
#ifndef SYS_TYPES_H
typedef int off_t;
#endif

#define BASENAMEF ".%s"
#define READ "r"
#define READ_BIN "r"
#define WRITE "w"
#define WRITE_BIN "w"
#define PATH_SEP "/"
#define PATH_C '/'
#define R_OK  4   /* File is readable */
#define W_OK  2   /* File is writeable */
#define F_OK  1   /* File is executable */
/*}}}  */
/*{{{  defines*/
#define SIGWINCH
#define _POSIX_PATH_MAX 120
#      define SEEK_SET           0
                               /* seek from the beginning of the file */
/*{{{  mouse and menu defines*/
#define AMIGA_MOUSE_TAG amiga
#define OS_MOUSE_TAG AMIGA_MOUSE_TAG
#define AMIGA_MOUSE_NAME   "amiga"
#define OS_MOUSE_NAME AMIGA_MOUSE_NAME
#define AMIGA_MOUSE_COUNT 30
#define OS_MOUSE_COUNT AMIGA_MOUSE_COUNT
#define AMIGA_MOUSE_NAMES {"LMBU", "S-LMBU","C-LMBU", "A-LMBU", "L-LMBU", \
   "R-LMBU", "LMBD", "S-LMBD","C-LMBD", "A-LMBD", "L-LMBD", "R-LMBD", \
   "RMB", "S-RMB", "C-RMB", "A-RMB", "L-RMB", "R-RMB", "MMBU", "S-MMBU", \
   "C-MMBU", "A-MMBU", "L-MMBU", "M-LMBU", "MMBD", "S-MMBD","C-MMBD", \
   "A-MMBD", "L-MMBD", "R-MMBD"}
#define OS_MOUSE_NAMES AMIGA_MOUSE_NAMES
#define OS_MOUSE_LG 0
#define OS_MOUSE_CODE ""
#define DEFAULT_MOUSE "amiga"
/*}}}  */
/*{{{  screen defines*/
#define Off_X (-4)       /* defines start position X-coordinate  */
#define Off_Y 2          /* .. Y-coordinate   in pixels */
#define Off_X_Right 7    /* defines space right in pixels */
#define Off_Y_Bottom 11  /* .. bottom */
#define Mul_X 8          /* defines character wide */
#define Mul_Y 9          /* defines character heigth */
#define Max_X 120        /* defines maximum columns */
#define Max_Y 50         /* defines maximum rows */
/*}}}  */
/*{{{  stdio defines*/
#  define eputs(a)  Print(a)
#  define oputs(a)  Print(a)
#  define pline(a)  Print(a)
#  define eputc(a)  oputc(a)
#  define oflush
/*}}}  */
/*{{{  KEYBOARD_C*/
#ifdef KEYBOARD_C
#  ifdef LATTICE
     int atexit (void (*)(void));
#  endif
#endif
/*}}}  */
/*{{{  GETMSG_C*/
#ifdef GETMSG_C
#  define I_SHELL_C
#endif
/*}}}  */
/*{{{  KEYBIND_C*/
#ifdef KEYBIND_C
#  define I_AMIGAFILEIO_C
#  define REF_TMP_FILE "T:kbrefXXXXXX"
#endif
/*}}}  */
/*{{{  FINDS_C*/
#ifdef FINDS_C
#  define I_SIGNALS_C
#endif
/*}}}  */

/*{{{  min*/
#ifndef min
#define min(a,b) ((a)<=(b)?(a):(b))
#endif
/*}}}  */
/*}}}  */
/*{{{  prototypes of amiga files*/
/*{{{  SIGNALS*/
#ifdef I_SIGNALS_C
  /* Prototypes for functions defined in signals.c */
  void handle_arexx_start (void);
  void handle_arexx_stop (void);
  void *my_calloc_ (unsigned int i, unsigned int j);
#ifdef LATTICE
  unsigned char *my_strstr (unsigned char *text, unsigned char *search);
#endif
#endif
/*}}}  */
/*{{{  KEYBOARD*/
#ifdef I_KEYBOARD_C
/* Prototypes for functions defined in keyboard.c */
  void scan_abort_key (void);
  unsigned char *getascnum (unsigned char *cptr, short *sptr);
  unsigned char **init_amiga (int *argc, struct WBStartup *argv);
  int reset_amiga (void);

  extern ULONG my_signal;
#  ifdef BOOL_H
  extern bool timer;
#  endif
#  ifdef DEVICES_TIMER_H
  extern struct timerequest tr;
#  endif
#  ifdef EXEC_PORTS_H
  extern struct MsgPort *mpp;
#  endif
#endif
/*}}}  */
/*{{{  SCREEN*/
#ifdef I_SCREEN_C
  /* Prototypes for functions defined in screen.c */
  void Set_Cursor (void);
  void Clear_Cursor (void);
  void Print (unsigned char *ptr);
  int oputc (unsigned char ch);
#  undef fflush
  int fflush (FILE *fp);
  int err_ausg (long i);
  void move_to_pos (void);
#endif
/*}}}  */
/*{{{  MKTEMP*/
#ifndef MKTEMP_C
  void mktemp ( unsigned char *str );
#endif
/*}}}  */
/*{{{  AMIGAFILEIO*/
#ifndef AMIGAFILEIO_C
  /* Prototypes for functions defined in amigafileio.c */
  FILE *popen (char *command, char *mode);
  int pclose (FILE *stream);
#endif
/*}}}  */
/*{{{  SHELL*/
#ifdef I_SHELL_C
  long my_lseek (int,long,int);
#endif
/*}}}  */
/*{{{  MISC*/
#ifdef MISC_C
#  define OS_FILEPROMPT 1
#else
  FILE *my_fopen (char *,char *);
  int my_fclose (FILE *);
#  ifdef LATTICE
  void sleep (int);
#  else
  char *stpblk (char *p);
#  endif
#endif
/*}}}  */
/*}}}  */
/*{{{  missing prototypes in origami.h*/
/*{{{  KEYTAB*/
#ifdef I_KEYTAB_C
  /*{{{  only if TOKEN is defined*/
#    ifdef KEYS_H
    TOKEN handle_click (int,int,int);
#    endif
  /*}}}  */
#endif
/*}}}  */
/*{{{  GETTK*/
#ifdef I_GETTK_C
#  ifdef KEYS_H
#  ifdef BOOL_H
  bool push_macro (TOKEN *);
#  endif
#  endif
#endif
/*}}}  */
/*{{{  MAIN*/
#ifndef MAIN_C
#ifndef KEYBIND_C
#ifndef VIEW_MAIN_C
int argparse  (int argc, char **argv, void *tabp, int tabsize);
#endif
#endif
#endif
/*}}}  */
/*{{{  STRERROR*/
char *strerror (int e);
/*}}}  */
/*}}}  */
/*{{{  additional define for some modules*/
/*{{{  MAIN*/
#ifdef MAIN_C
#ifdef ORIGAMI
#  undef exit
#  define exit(s)   err_ausg(s)
#  define os_init() argv = init_amiga(&argc, (struct WBStartup *)argv)
#endif
#endif
/*}}}  */
/*{{{  LOOP*/
#ifdef LOOP_C
#  ifdef AREXX
#    include "origami:amiga/h/origami_arexx.h"
     extern bool arexx_tag;
     extern int arexx_var;
#    define sighan1 if (arexx_tag) handle_arexx_start(); \
            if (macro_int[arexx_var]!=-1) handle_arexx_stop();
#  else
#    define sighan1
#  endif
#  define signal_handler sighan1
#endif
/*}}}  */
/*{{{  KEYBOARD*/
#ifdef KEYBOARD_C
#  define PAL_SCREEN_HEIGHT  256
#  define NTSC_SCREEN_HEIGHT 200
#  define PAL_TYPE  FALSE
#  define NTSC_TYPE TRUE

#  ifdef PAL
#    define DEFAULT_SCREEN_HEIGHT   PAL_SCREEN_HEIGHT
#    define PAL_NTSC_DEFAULT        PAL_TYPE
#  else
#    define DEFAULT_SCREEN_HEIGHT   NTSC_SCREEN_HEIGHT
#    define PAL_NTSC_DEFAULT        NTSC_TYPE
#  endif

#  define CSI 0x9B
#  define buffer_keys_max  200
#  define CONU_LIBRARY -1
#  define CSI_RAW_KEY 69
#  define CSI_NUMBER_PAD 70
#  define CSI_MENU_PICK 71

#endif
/*}}}  */
/*{{{  GETTK*/
#ifdef GETTK_C
#  undef signal
#  define signal(a,b)  {}
#  define chartest if (ch=='\0' || ch=='\n' || (!hard_tab && ch=='\t')) ch=O_NOP;
#endif
/*}}}  */
/*{{{  GETMSG*/
#ifdef GETMSG_C
#      undef lseek
#      define lseek(a,b,c)       my_lseek(a,b,c)
                               /* problems with lseek behind file */
#endif
/*}}}  */
/*{{{  SCANNER*/
#ifdef SCANNER_C
#  define OS_USINGS  use ("amiga");
#endif
/*}}}  */
/*{{{  KEYBIND*/
#ifdef KEYBIND_C
  /*{{{  write binding to .origamibind*/
#  define bnd_wr(kbd,name,code) \
    fprintf(bnd,kbd==mouse_kbd?"%d|%-35.*sMouse: %c\n":"%d|%-35.*s%s\n", \
                kbd,strlen(name),name,code)
  /*}}}  */
  /*{{{  SORTCMD*/
#    define SORTCMD "sort colstart 3 to %s"
  /*}}}  */
#      define SEEK_SET           0
#endif
/*}}}  */
/*{{{  KEYBIND || MAIN*/
#if (defined(KEYBIND_C)||defined(MAIN_C)||defined(VIEW_MAIN_C)||defined(INIT_C))
#  define homecheck(x)
#  define STD_BASENAME ".origami"
#  undef PATH_SEP
#  if (defined(I_INIT_C)||defined(KEYBIND_C)||defined(VIEW_MAIN_C))
#    define PATH_SEP  (strchr (":/",s[strlen(s)-1]) ? "" : "/")
#  else
#    define PATH_SEP  (strchr (":/",keyfile[strlen(keyfile)-1]) ? "" : "/")
#  endif
#endif
/*}}}  */
/*{{{  INIT*/
#ifdef INIT_C
# ifdef AREXX
#  undef calloc
#  define calloc(a,b)    my_calloc_(a,b)
# endif
#endif
/*}}}  */
/*{{{  FINDS*/
#ifdef FINDS_C
#ifdef LATTICE
#  undef strstr
#  define strstr(a,b)   my_strstr(a,b)
   /* This has to be done cause an SAS C Error ! */
#endif
#endif
/*}}}  */
/*{{{  VIRTUAL*/
#  ifdef VIRTUAL_C
#    ifdef VIRTUAL
#      define BLOCK_SIZE         90
                               /* mininum is 4*sizeof(int)                */
                               /* maximum is MAX_FIELD_SIZE+sizeof(int)   */
#      define ORI_TMP            "T:.oritmpXXXXXXXXX"
#      define FIXED              8
                               /* number of secure lines in memory        */
                               /* the minum is 8 (number of lines needed  */
                               /* for internal expressions). You can use  */
                               /* greater values (screensize for example) */
#      define SEEK_SET           0
                               /* seek from the beginning of the file */
#      undef lseek
#      define lseek(a,b,c)       my_lseek(a,b,c)
                               /* problems with lseek behind file */
#    endif
#  endif
/*}}}  */
/*{{{  ORIEDT*/
#  ifdef ORIEDT_C
#    define BASIC_ELEMENTS    256   /* number of first element's block */
#    define NEW_ELEMENT_LIMIT  16   /* minimum number of malloced      */
                                  /* element's block.                */
                                  /* if malloc cannot return a block */
                                  /* of NEW_ELEMENT_LIMIT elements,  */
                                  /* a `not enough memory' warning   */
                                  /* appears!                        */
#    define NEW_ELEMENT_MAX    64   /* maximum number of malloced      */
                                  /* element's block (used if lines  */
                                  /* are handled virtuell)           */
#  endif
/*}}}  */
/*{{{  FOLDFILLING*/
#  ifdef FOLDFILING_C
#     define ORICORE_NAME "oricoreXXXXXXXX"
#  endif
/*}}}  */
/*{{{  SET || PARSECOND || MAIN*/
#  if defined(SET_C) || defined(PARSECOND_C) || defined(MAIN_C)
#    define SETPART 8
#    define SETSIZE (256/SETPART)
#  endif
/*}}}  */
/*{{{  WRITEFOLDS*/
#ifdef WRITEFOLDS_C
#  undef fopen
#  undef fclose
#  define fopen(a,b) my_fopen(a,b)
#  define fclose(a)  my_fclose(a)
#endif
/*}}}  */
/*}}}  */
