/*{{{  #includes*/
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include <dirent.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <local/bool.h>

#include "../h/keys.h"
#include "../h/envvar_str.h"

#define GETRC
#define I_RC_CODE

#include "fold_os.h"
#include "../h/rcformat.h"
/*}}}  */

/*{{{  variables*/
/*{{{  prototypes for getopt*/
int getopt(int,char**,char*);
extern char *optarg;
extern int optind;
/*}}}  */
/*{{{  fold-marks*/
char fold_open_str[tag_length]=begin_standard;
char fold_close_str[tag_length]=end_standard;
char fold_file_str[tag_length]=file_standard;
char fold_line_str[tag_length]=line_standard;
/*}}}  */
/*{{{  for commandline-options*/
char *keybase="";
char *mark_mode="";
bool always=FALSE;
bool mark_no_perm=FALSE;
/*}}}  */
FILE *rcfile;
int depth=255;
int user;
int group;
/*}}}  */

/*{{{  set_other_mark*/
/*{{{  completebase*/
char *completebase(char *s, char *t)
{ if (*keybase)
   /*{{{  use given prefix*/
     strcpy(s,keybase);
   /*}}}  */
  else
   /*{{{  use home/KEYBASE*/
   { char *h=getenv(HOME);

     if (h==NULL)
        *s='\0';
     else
      { strcpy(s,h);
        strcat(s,PATH_SEP);
        strcat(s,KEYBASE);
      }
   }
   /*}}}  */
  strcat(s,t);
  return(s);
}
/*}}}  */
/*{{{  parse rcfile*/
bool rc_file_parser(void)
{
  readtags input;

  rc_get_w(rcfile);
  while ((input=rc_get_c(rcfile))!=RC_ENDE && input!=RC_BIND) {
    switch (input) {
      /*{{{  termalias*/
      case RC_TERMALIAS:
       { int ch;

         while ((ch=rc_get_c(rcfile)) && ch!=EOF)
            do ch=rc_get_c(rcfile); while (ch && ch!=EOF);
         do ch=rc_get_c(rcfile); while (ch && ch!=EOF);
         break;
       }
      /*}}}  */
      /*{{{  keydef*/
      case RC_DEFKEY: {
        int nodes;

        rc_get_c(rcfile);
        while (rc_get_c(rcfile));
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        /*{{{  get size of table*/
        nodes=rc_get_w(rcfile);
        /*}}}  */
        /*{{{  read all nodes*/
        while (nodes--) {
          rc_get_w(rcfile);
          rc_get_c(rcfile);
          rc_get_w(rcfile);
          rc_get_w(rcfile);
        }
        /*}}}  */
        break;
      }
      /*}}}  */
      /*{{{  usermodestrings*/
      case RC_MODE: {
        int lg;

        rc_get_c(rcfile);
        lg=rc_get_w(rcfile);
        if (lg) while (lg--) rc_get_c(rcfile);
        lg=rc_get_w(rcfile);
        if (lg) while (lg--) rc_get_c(rcfile);
        break;
      }
      /*}}}  */
      /*{{{  macrodef/init*/
      case RC_INITMACRO:
      case RC_DEFMACRO: {
        int lg;

        rc_get_w(rcfile);
        lg=rc_get_w(rcfile);
        while (lg--) rc_get_w(rcfile);
        break;
      }
      /*}}}  */
      /*{{{  keyname*/
      case RC_KEYNAME: {
        int i=keyn_lg;

        while (i--) rc_get_c(rcfile);
        break;
      }
      /*}}}  */
      /*{{{  ints,macs,macro-hooks and vars*/
      case RC_VMAC:
      case RC_KNBMAC:
      case RC_PROMPTOUT:
      case RC_PROMPTIN:
      case RC_ABORTM:
      case RC_AUTOM:
      case RC_INTS:
      case RC_MACCOUNT:
        rc_get_w(rcfile);
        break;
      case RC_S_VAR:
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        break;
      /*}}}  */
      /*{{{  sets*/
      case RC_CHARSET:
       { int lg=(sizeof(Set)*rc_get_w(rcfile))>>5;

         while (lg--) rc_get_c(rcfile);
         break;
       }
      /*}}}  */
      /*{{{  nostat*/
      case RC_NO_TITLE:
         break;
      /*}}}  */
      /*{{{  mouse-vars*/
      case RC_M_VAR:
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        rc_get_w(rcfile);
        break;
      /*}}}  */
      /*{{{  mouse-tab*/
      case RC_M_TAB: {
        int i;

        for (i=rc_get_w(rcfile);i--;rc_get_w(rcfile));
        break;
      }
      /*}}}  */
      /*{{{  break*/
      case RC_BREAK: {
        rc_get_c(rcfile);
        break;
      }
      /*}}}  */
      /*{{{  marks*/
      case RC_MARKS: {
        char name[mark_name_lg];
        int i;

        /*{{{  read stored name*/
        for (i=0;i<(mark_name_lg-1);i++)
          name[i]=rc_get_c(rcfile);
        name[mark_name_lg-1]='\0';
        /*}}}  */
        if (!strcmp(mark_mode,name)) {
          /*{{{  reset mark-strings*/
          *mark_mode='\0';
          for (i=0;i<(tag_length-1);i++) fold_open_str[i]=rc_get_c(rcfile);
          for (i=0;i<(tag_length-1);i++) fold_line_str[i]=rc_get_c(rcfile);
          for (i=0;i<(tag_length-1);i++) fold_file_str[i]=rc_get_c(rcfile);
          for (i=0;i<(tag_length-1);i++) fold_close_str[i]=rc_get_c(rcfile);
          /*}}}  */
        } else
          /*{{{  overread the data*/
          for (i=0;i<(4*(tag_length-1));i++,rc_get_c(rcfile));
          /*}}}  */
        break;
      }
      /*}}}  */
      /*{{{  error!*/
      default: return(TRUE);
      /*}}}  */
    }
  }
  /*{{{  if marks needed and not found -> error*/
  if (*mark_mode) {
    printf("no matching marks found.\n");
    return(TRUE);
  }
  /*}}}  */
  return(FALSE);
}
/*}}}  */

int set_other_mark(void)
{
  char keyfile[_POSIX_PATH_MAX+1];

  if (getenv(HOME)==NULL) {
    printf ("no home for keybase\n");
    return (1);
  }
  /*{{{  open rc-file*/
  if ((rcfile = fopen(completebase(keyfile,"rc"),"r")) == 0) {
    printf ("no keybase %s accessable.\n",keyfile);
    return (1);
  }
  /*}}}  */
  if (rc_file_parser()) {
    printf("incorrect rc-data.\n");
    return(1);
  }
  fclose(rcfile);
  return (0);
}
/*}}}  */
/*{{{  path_get*/
char *path_get(char *dir)
{
  char *s, buf[_POSIX_PATH_MAX+1];

  if ((s=getcwd(buf, _POSIX_PATH_MAX))==NULL) {
    perror("pwd");exit(1);
  }
  return(strcpy(dir,s));
}
/*}}}  */
/*{{{  dirhandle*/
/*{{{  dir_ent_cmp*/
int dir_ent_cmp(char **a,char **b)
{
  return(strcmp(*a,*b));
}
/*}}}  */
/*{{{  forward for filehandle*/
void filehandle(char *name, char *dir, int curr_depth);
/*}}}  */

void dirhandle(char *name,char *dir,int curr_depth)
{
  char new_wd[_POSIX_PATH_MAX+1];
  DIR *dirptr;

  if (curr_depth<=depth) {
    /*{{{  recursion*/
    if (chdir(name))
      printf("%s cannot enter\n",name);
    else {
      /*{{{  directory entered*/
      if ((dirptr=opendir(CURR_DIR))!=NULL) {
        /*{{{  can read the entries*/
        /*{{{  local variables*/
        struct dirent *ls;
        char **s;
#                define bl_size 128
        int count=0;
        int curr;
        int curr_size=bl_size;
        bool marked_point=FALSE;
        /*}}}  */

        /*{{{  get current path*/
        if (path_get(new_wd)==NULL) {
          perror("pwd");exit(1);
        }
        /*}}}  */
        /*{{{  get buffer for dir-entries*/
        if ((s=(char**)malloc(curr_size*sizeof(char*)))==NULL) {
          fprintf(stderr,"no memory\n");
          exit(1);
        }
        /*}}}  */
        /*{{{  read the contents of the directory*/
        while ((ls=readdir(dirptr))!=NULL) {
          /*{{{  expand entry-list*/
          if (count==curr_size) {
            char **old=s;
            int x=0;

            if ((s=(char**)malloc((curr_size+bl_size)*sizeof(char*)))==NULL) {
              fprintf(stderr,"no memory\n");
              exit(1);
            }
            while (x<curr_size) {s[x]=old[x];x++;}
            free(old);
            curr_size+=bl_size;
          }
          /*}}}  */
          /*{{{  get memory for entry*/
          if ((s[count]=malloc(strlen(ls->d_name)+1))==NULL) {
            fprintf(stderr,"no memory\n");
            exit(1);
          }
          /*}}}  */
          strcpy(s[count++],ls->d_name);
        }
        /*}}}  */
        qsort((void*)s,(size_t)count,(size_t)sizeof(char*),dir_ent_cmp);
        printf("%s  %s\n",fold_open_str,name);
        /*{{{  handle all entries of the directory*/
        curr=0;
        while (curr<count) {
          if (strcmp(CURR_DIR,s[curr]) && strcmp(PREV_DIR,s[curr])) {
            /*{{{  maybe mark .???*/
            if (!marked_point && s[curr][0]=='.') {
              marked_point=TRUE;
              printf("%s  .xxx files\n",fold_open_str);
            }
            /*}}}  */
            /*{{{  maybe endmark .???*/
            if (marked_point && s[curr][0]!='.') {
              marked_point=FALSE;
              printf("%s  \n",fold_close_str);
            }
            /*}}}  */
            filehandle(s[curr],new_wd,curr_depth);
            free(s[curr]);
          }
          curr++;
        }
        /*}}}  */
        if (marked_point) printf("%s  \n",fold_close_str);
        printf("%s  \n",fold_close_str);
        free(s);
        closedir(dirptr);
        /*}}}  */
      } else
        printf("cannot read this directory.\n");
      /*}}}  */
    }
    chdir(dir);
    /*}}}  */
  } else
    printf("maxdepth reached for %s.\n",name);
}
/*}}}  */
/*{{{  filehandle*/
void filehandle(char *name,char *dir,int curr_depth)
{
  struct stat buff;
  int stat_ret;

  if (curr_depth<=depth)
    if (stat_ret=stat(name,&buff))
      printf("cannot stat %d: %s.\n",stat_ret,name);
    else {
      if (   ((buff.st_uid==user) && (S_IRUSR&buff.st_mode))
          || ((buff.st_gid==group) && (S_IRGRP&buff.st_mode))
          || (S_IROTH&buff.st_mode)
          || always )
      {
        /*{{{  handle a directory*/
        if (S_ISDIR(buff.st_mode)) dirhandle(name,dir,curr_depth+1);
        /*}}}  */
        /*{{{  handle a file*/
        if (S_ISREG(buff.st_mode)) {
          printf("%sF %s\n",fold_open_str,name);
          printf("%sF %s%s%s\n",fold_file_str,dir,PATH_SEP,name);
          printf("%s  \n",fold_close_str);
        }
        /*}}}  */
      } else if (mark_no_perm)
        printf("no permission for %s\n",name);
    }
}
/*}}}  */

/*{{{  main*/
int main(argc, argv) int argc; char *argv[];
{ char call_path[_POSIX_PATH_MAX+1];
  int no;

  user=getuid();
  group=getgid();
  /*{{{  parse arguments*/
  { int c;

    while ((c=getopt(argc,argv,"k:m:hd:ap"))!=EOF)
    switch (c)
    {
      /*{{{  d*/
      case 'd':
        depth=atoi(optarg);
        if (depth<=0)
         { printf("nothing to do.\n");exit(0); }
        break;
      /*}}}  */
      /*{{{  m*/
      case 'm': mark_mode=optarg;break;
      /*}}}  */
      /*{{{  a*/
      case 'a': always=TRUE;break;
      /*}}}  */
      /*{{{  p*/
      case 'p': mark_no_perm=TRUE;break;
      /*}}}  */
      /*{{{  k*/
      case 'k': keybase=optarg;break;
      /*}}}  */
      /*{{{  h and error*/
      case 'h':
      case '?':
      default:
        fprintf(stderr,"Usage: dirfold [-ahp] [-d depth] [-k namebase] [-m markname] [filename] ...\n");
        exit(0);
      /*}}}  */
    }
  }
  /*}}}  */
  if (*mark_mode && set_other_mark()) exit(1);
  path_get(call_path);
  if (argv[no=optind])
    do
       filehandle(argv[no],call_path,0);
    while (argv[++no]);
  else
    filehandle(CURR_DIR,call_path,0);
  return(0);
}
/*}}}  */
