/*{{{  possible switches*/
/*
   Use the following configuration symbols to generate a makefile for
   your system with "/usr/lib/cpp -DCONFIG Makefile.cpp >Makefile"

     SUNOS_GCC
     HPUX_ANSI
     LINUX
   Set the variables for unproto to your version of it.
     SUNOS_UNPROTO_CC
     HPUX_UNPROTO
   The following defines are not tested. This version must be compiled with
   ANSI-cc or with uncompile!
     MINIX_BCCi386  (untested)
     MINIX_GCCi386  (untested)
     MIPS           (untested)
*/
/*}}}  */
/*{{{  SUNOS*/
/*{{{  UNPROTO_CC*/
#ifdef SUNOS_UNPROTO_CC
CC=             cc
CPP_DIR=        /home/balu/ourbin/bin.$(ARCHITECTURE)
CFLAGS=         -O4 -Qpath $(CPP_DIR) -I.. -D_POSIX_SOURCE
LDFLAGS=
LIBS=           -ltermcap
O=              o
AWK=            awk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif
#endif
/*}}}  */
/*{{{  SUNOS_GCC*/
#ifdef SUNOS_GCC
CC=             gcc
CFLAGS=         -O -ansi -pedantic -Wall -I.. -D_POSIX_SOURCE \
		-D_POSIX_PATH_MAX=256
LDFLAGS=
LIBS=           -ltermcap
O=              o
AWK=            awk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif
#endif
/*}}}  */
/*}}}  */
/*{{{  HPUX*/
/*{{{  HPUX_UNPROTO*/
#ifdef HPUX_UNPROTO
CC=             cc
CPP=/home/balu/ourbin/bin.hp/cpp
CFLAGS=         -O -tp,$(CPP) -I.. -D_POSIX_SOURCE
LDFLAGS=
LIBS=           -ltermcap
O=              o
AWK=            awk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif
#endif
/*}}}  */
/*{{{  HPUX_ANSI*/
#ifdef HPUX_ANSI
CC=             cc
CFLAGS=         +O3 -Aa -I.. -D_POSIX_SOURCE
LDFLAGS=
LIBS=           -ltermcap
O=              o
AWK=            awk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif
#endif
/*}}}  */
/*}}}  */
/*{{{  LINUX*/
#ifdef LINUX
CC=             gcc
CFLAGS=         -O -Wall -I.. -D_POSIX_SOURCE
LDFLAGS=
LIBS=           -ltermcap
O=              o
AWK=            gawk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   /usr/local/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       /usr/local/bin
#endif
#endif
/*}}}  */
/*{{{  MINIX*/
/*{{{  BCCI386*/
#ifdef MINIX_BCCi386
CC=             bcc -3
CFLAGS=         -O -D_MINIX -D_POSIX_SOURCE
LDFLAGS=
LIBS=
O=              o
AWK=            bawk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   /usr/local/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       /usr/local/bin
#endif
#endif
/*}}}  */
/*{{{  GCCi386*/
#ifdef MINIX_GCCi386
CC=             gcc
CFLAGS=         -O -ansi -pedantic -D_MINIX -D_POSIX_SOURCE -Wall
LDFLAGS=
LIBS=
O=              o
AWK=            awk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   /usr/local/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       /usr/local/bin
#endif
#endif
/*}}}  */
/*}}}  */
/*{{{  MIPS*/
#ifdef MIPS
CC=             /posix/bin/cc
CFLAGS=         -O -I.. -D_POSIX_SOURCE
LDFLAGS=
LIBS=           -L/usr/lib -ltermcap
O=              o
AWK=            awk
#ifndef ORIGAMI_PATH
ORIGAMI_PATH=   $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin
#endif
#endif
/*}}}  */

HDRS=           ../h/keys.h ../h/rcformat.h fold_os.h ../h/token.h \
		../h/envvar_str.h

SRCS=           main.c

OBJS=           main.$(O)

dirfold:        $(OBJS)
		$(CC) $(LDFLAGS) -o $@ $(OBJS) $(LIBS)

../h/token.h:   ../h/tokens ../h/tok2h.awk
		$(AWK) -f ../h/tok2h.awk ../h/tokens >../h/token.h

$(OBJS):        $(HDRS)

install:        dirfold
		install -d $(BIN_PATH)
		install dirfold $(BIN_PATH)
		strip $(BIN_PATH)/dirfold

deinstall:
		rm -f $(BIN_PATH)dirfold

clean:
		rm -f core symbol.out err.out $(OBJS)

clobber:        clean
		rm -f dirfold Makefile ../h/token.h
