/*{{{  #includes*/
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <local/bool.h>

#define CODEHELP_C

#include "keybind.h"
/*}}}  */

/*{{{  variables*/
#define DV_MAX (var_count>>2)
public bool new_vars_enabled=FALSE;
private int dv_list[DV_MAX];
private bool dv_init=FALSE;
/*}}}  */

/*{{{  end_parse*/
public bool end_parse(char *er)
{
  if (get_token(FALSE)!=END) {
    error_po();
    fprintf(stderr,er);

    return(TRUE);
  }

  return(FALSE);
}
/*}}}  */
/*{{{  begin_parse*/
public bool begin_parse(void)
{
  if (get_token(FALSE)!=BEGIN) {
    error_po();
    fprintf(stderr,M_WANTBEGIN);

    return(TRUE);
  }

  return(FALSE);
}
/*}}}  */
/*{{{  parse a keycode sequence*/
public bool parse_keysequence(K_SEQ *k)
{
  char *c[termmax];
  char *p;
  int i;
  tokens token;

  if (begin_parse()) return(TRUE);
  /*{{{  init the pointers*/
  p=k->prot;
  for (i=0;i<termmax;c[i]=k->codes[i],i++);
  /*}}}  */
  /*{{{  read and handle 'til )*/
  do {
    token=get_token(TRUE);
    switch (token) {
      /*{{{  HEX-handling*/
      case HEX:
        for (i=0;i<termmax;i++)
          if ((!i || i<k->count) && c[i]) *c[i]++=tk_char;
        /*{{{  generate protocol of this binding*/
        strcpy(p,tk_string);
        while (*p) p++;
        /*}}}  */
        break;
      /*}}}  */
      /*{{{  CHAR-handling*/
      case CHAR:
        for (i=0;i<termmax;i++)
          if ((!i || i<k->count) && c[i]) *c[i]++=tk_char;
        /*{{{  generate protocol of this binding*/
        if (tk_char==127)      {*p++='C';*p++='-';*p++='?';}
        else if (tk_char>=' ') *p++=tk_char;
        else                   {*p++='C';*p++='-';*p++=tk_char+'A'-1;}
        *p++=' ';
        /*}}}  */
        break;
      /*}}}  */
      /*{{{  META-handling*/
      case META:
        for (i=0;i<termmax;i++)
          if ((!i || i<k->count) && c[i]) *c[i]++=27;
        *p++='M';*p++='-';
        break;
      /*}}}  */
      /*{{{  ALIAS-handling*/
      case DOLLAR: {
         { ALIAS **a=func_key;

           while (*a && strcmp((*a)->key_name,tk_string)) a++;
           if (*a)
            /*{{{  alias found -> normal alias, copy to all sequences*/
            { char *s=(*a)->code;
              int  lg=(*a)->length;

              for (i=0;i<termmax;i++,lg=(*a)->length,s=(*a)->code)
                 if ((!i || i<k->count) && c[i])
                    while (lg--) *c[i]++ = *s++;
            }
            /*}}}  */
           else
            /*{{{  handle as terminal-alias*/
            { bool tf=FALSE;

              for (i=0;i<k->count;i++)
                 if (c[i])
                  /*{{{  handle a terminal-alias*/
                  { char an[name_lg];

                    tf=TRUE;
                    /*{{{  create correct name*/
                    strcpy(an,k->terms[i]);
                    strcat(an,"@");
                    strcat(an,tk_string);
                    /*}}}  */
                    a=func_key;
                    while (*a && strcmp((*a)->key_name,an)) a++;
                    if (*a)
                     /*{{{  alias found*/
                     { char *s=(*a)->code;
                       int  lg=(*a)->length;

                       while (lg--) *c[i]++ = *s++;
                     }
                     /*}}}  */
                    else
                     /*{{{  don't bind this term-alias!*/
                     { if (verbose)
                        { error_po();fprintf(stderr,F_MISALIAS,an); }
                       c[i]=0;
                     }
                     /*}}}  */
                  }
                  /*}}}  */
              if (!tf)
               /*{{{  error*/
               { error_po();
                 fprintf(stderr,F_NOALIAS,tk_string);
                 return(TRUE);
               }
               /*}}}  */
            }
            /*}}}  */
           strcpy(p,tk_string);
           while (*p) p++;
           *p++=' ';
         }
      }
      /*}}}  */
      case END: break;
      case ERROR: return(TRUE);
      /*{{{  default=error*/
      default: {
        error_po();
        fprintf(stderr,M_WANTEND);
        return(TRUE);
      }
      /*}}}  */
    }
  } while (token!=END);
  /*}}}  */
  *p='\0';
  for (i=0;i<termmax;i++) k->lg[i]=c[i]?c[i]-k->codes[i]:-1;
  return(FALSE);
}
/*}}}  */
/*{{{  name_to_var transform token NAME to token VARIABLE, if possible*/
public tokens name_to_var(tokens token, bool newvars)
{ static VARS x;

  if (token==NAME && newvars)
   { (tk_var= &x)->no=creat_var(tk_string);return(VARIABLE); }
  return(token);
}
/*}}}  */
/*{{{  put variable-adress*/
public TOKEN *put_var(TOKEN *buff)
{
  tokens token=name_to_var(get_token(FALSE),new_vars_enabled);

  switch (token) {
    case VARIABLE:
      *buff++ = (TOKEN) tk_var->no;
      break;
    case ERROR:
      return(0);
      break;
    default:
      error_po();
      fprintf(stderr,M_NOVAR);
      return(0);
  }
  return(buff);
}
/*}}}  */
/*{{{  dummy_var*/
public int dummy_var(int no)
{
  /*{{{  maybe init it*/
  if (!dv_init)
   { int i;
     for (i=0;i<DV_MAX;dv_list[i++]= -1);
     dv_init=TRUE;
   }
  /*}}}  */
  /*{{{  no dummy-vars available -> error*/
  if (no>=DV_MAX)
   { error_po();
     fprintf(stderr,M_NOVAS);
     exit(1);
   }
  /*}}}  */
  if (dv_list[no]==-1)
   { char s[10];

     sprintf(s,"(dummy%d)",no);
     return(dv_list[no]=creat_var(s));
   }
  else
     return(dv_list[no]);
}
/*}}}  */
/*{{{  create a jump-command*/
public TOKEN *generate_jmp(TOKEN j, TOKEN *ad, TOKEN *dest)
{
  switch (j)
   { case M_JMP:
     case M_CALL:
     case M_JMP_TRUE:
     case M_JMP_FALSE:
        break;
     default:
        fprintf(stderr,M_NOJUMP);
        return(0);
   }
  *ad++ = j;
  *ad= dest-(ad+1);
  return(ad+1);
}
/*}}}  */
/*{{{  opt_mac*/
#ifndef NO_OPTI
   /*{{{  next_command*/
#   include "codelg.h"

   private TOKEN *next_command(TOKEN *x)
   { if (*x>O_EXE_MACRO || *x<O_NOP) return(x+1);
     switch (cmd_type[*x-O_NOP])
      { case COM_II:
           x++;
        case COM_I:
        case COM_C:
        case COM_A:
           return(x+2);
        case COM_IIP:
           x++;
        case COM_IP:
           x++;
        case COM_P:
           x++;
           while (*x!=M_END_MACRO)
              if (*x==M_INT_STRING) x+=2; else x++;
        case COM:
           return(x+1);
        default:
           fprintf(stderr,"CRASH\n");
           exit(1);
      }

   return(0);
   }
   /*}}}  */
   /*{{{  jmp_dest*/
   private TOKEN *jmp_dest(TOKEN *x)
   { return(x+2+*(x+1));
   }
   /*}}}  */
   /*{{{  isdummy*/
   private bool isdummy(int x)
   { int i;

     if (dv_init) for (i=0;i<DV_MAX;i++) if (dv_list[i]==x) return(TRUE);
     return(FALSE);
   }
   /*}}}  */

   public TOKEN *opt_mac(TOKEN *start,TOKEN *end,bool no_not_opt)
   { bool changed;
     int loop,jmp_cut,modified,shrink;
     TOKEN buff[macro_lg];

     loop=jmp_cut=shrink=modified=0;
     do
      /*{{{  one optimisation step*/
      { changed=FALSE;
        /*{{{  start->buff, jmp opt*/
        { TOKEN *current,*dest,*next,*jmp;

          for (current=start,dest=buff;current!=end;)
           { bool inverse=FALSE;

             next=next_command(current);
             switch (*current)
              { case M_CALL:
                case M_JMP:
                 /*{{{  cut simple jmp-sequences*/
                 { TOKEN typ= *current;

                   for (jmp=jmp_dest(current);jmp!=end;)
                    { switch (*jmp)
                       { case O_NOP:
                            jmp++;changed=TRUE;jmp_cut++;continue;
                         case M_JMP:
                            jmp=jmp_dest(jmp);changed=TRUE;jmp_cut++;continue;
                         case M_NOT:
                            if (no_not_opt) break;
                            fprintf(stderr,"JMP-NOT optimizer crash\n");
                            no_not_opt=TRUE;
                            current=start;
                            dest=buff;
                            continue;
                         default:
                            break;
                       }
                      break;
                    }
                   dest=generate_jmp(typ,dest,buff+(jmp-start));
                   current=next;
                   continue;
                 }
                 /*}}}  */
                case M_NOT:
                 /*{{{  break, or prefix for jmp-condition*/
                   if (no_not_opt)
                    { *dest++ = *current++;continue; }
                   else if (*next!=M_JMP_FALSE && *next!=M_JMP_TRUE)
                    { fprintf(stderr,"JMP-NOT optimizer crash\n");
                      no_not_opt=TRUE;
                      current=start;
                      dest=buff;
                      continue;
                    }
                   inverse=TRUE;
                   changed=TRUE;
                   jmp_cut++;
                   *dest++ = O_NOP;
                   current=next;
                   next=next_command(current);
                 /*}}}  */
                case M_JMP_TRUE:
                case M_JMP_FALSE:
                 /*{{{  handle jmp-sequences, value of tag is known*/
                 { TOKEN typ;
                   bool tag_value;

                   tag_value= *current==M_JMP_TRUE;
                   typ=tag_value?(inverse?M_JMP_FALSE:M_JMP_TRUE)
                                :(inverse?M_JMP_TRUE:M_JMP_FALSE);
                   for (jmp=jmp_dest(current);jmp!=end;)
                    { switch (*jmp)
                       { case O_NOP:
                            jmp++;changed=TRUE;jmp_cut++;continue;
                         case M_JMP:
                            jmp=jmp_dest(jmp);changed=TRUE;jmp_cut++;continue;
                         case M_NOT:
                            if (no_not_opt) break;
                            tag_value = !tag_value;
                            jmp+=1;
                            jmp_cut++;
                            changed=TRUE;
                            continue;
                         case M_JMP_FALSE:
                            if (tag_value) jmp+=2; else jmp=jmp_dest(jmp);
                            changed=TRUE;
                            jmp_cut++;
                            continue;
                         case M_JMP_TRUE:
                            if (tag_value) jmp=jmp_dest(jmp); else jmp+=2;
                            changed=TRUE;
                            jmp_cut++;
                            continue;
                         default:
                            break;
                       }
                      break;
                    }
                   dest=generate_jmp(typ,dest,buff+(jmp-start));
                   current=next;
                   continue;
                 }
                 /*}}}  */
                default:
                 /*{{{  copy the code*/
                   for (;current!=next;*dest++ = *current++);
                   continue;
                 /*}}}  */
              }
           }
        }
        /*}}}  */
        /*{{{  buff->start, shrink and modify to better code*/
        { TOKEN *current,*next,*dest,*opt_end;

            /*{{{  cut leading O_NOP*/
            while (buff[0]==O_NOP && end!=start)
             { TOKEN *x;

               for (x=buff;x!=(buff+(end-start));*x=*(x+1),x++);
               end--;
               changed=TRUE;
               shrink++;
             }
            /*}}}  */
            /*{{{  mark reachable commands, opt_end= &(good M_END_MACRO)))*/
            for (current=start,opt_end=0;current!=end;*current++ = O_NOP);
            for (dest=end,start[0]=0;;)
               if (dest==end)
                /*{{{  look for next reached, unhandled command*/
                { for (dest=start;dest!=end;) if (*dest==0) break; else dest++;
                  if (dest==end) break;
                  current=buff+(dest-start);
                  continue;
                }
                /*}}}  */
               else
                /*{{{  handle a reachable, unhandled command*/
                  switch (*current)
                   { case O_NOP:
                        current++;dest++;continue;
                     case M_CALL:
                     case M_JMP:
                     case M_JMP_TRUE:
                     case M_JMP_FALSE:
                      { TOKEN *jmp;

                        jmp=start+(jmp_dest(current)-buff);
                        if (jmp!=end && *jmp!=M_END_MACRO) *jmp=0;
                      }
                     default:
                      { bool no_cont;

                        /*{{{  set no_cont to correct value*/
                        switch (*current)
                         { case M_EXIT:
                           case M_END_MACRO:
                           case M_JMP:
                              no_cont=TRUE;
                              break;
                           default:
                              no_cont=FALSE;
                         }
                        /*}}}  */
                        next=next_command(current);
                        if (*dest==M_EXIT) opt_end=0;
                        for (;current!=next;)
                         { if (*current++==M_END_MACRO && !opt_end) opt_end=dest;
                           *dest++ = M_END_MACRO;
                         }
                        if (no_cont) dest=end;
                        continue;
                      }
                   }
                /*}}}  */
            /*}}}  */
            /*{{{  shrink code*/
            for (current=start,dest=buff;current!=end;)
               if (*current==M_END_MACRO)
                { *current++ = *dest++;continue; }
               else
                { int i,limit,max;

                  limit=current-start;
                  max=end-start;
                  /*{{{  modify jmp pre -> atfer*/
                  for (i=0;i<limit;)
                     switch (buff[i])
                      { case M_CALL:
                        case M_JMP:
                        case M_JMP_TRUE:
                        case M_JMP_FALSE:
                           if (jmp_dest(buff+i)>(buff+limit))
                            { start[i+1]--;buff[i+1]--; }
                        default:
                           i=next_command(buff+i)-buff;
                      }
                  /*}}}  */
                  /*{{{  modify atfer -> pre*/
                  for (i=limit+1;i!=max;)
                     switch (buff[i])
                      { case M_CALL:
                        case M_JMP:
                        case M_JMP_TRUE:
                        case M_JMP_FALSE:
                           if (jmp_dest(buff+i)<(buff+limit)) buff[i+1]++;
                        default:
                           i=next_command(buff+i)-buff;
                      }
                  /*}}}  */
                  if (opt_end && (opt_end-start)>limit) opt_end--;
                  end--;
                  for (i=limit;i!=max;start[i]=start[i+1],buff[i]=buff[i+1],i++);
                  changed=TRUE;
                  shrink++;
                  continue;
                }
            /*}}}  */
            /*{{{  modify some simple statement-sequences*/
            /*{{{  all jmp's to M_END_MACRO to opt_end!*/
            if (opt_end)
             { for (current=start;current!=end;)
                  switch (*current)
                   { case M_CALL:
                     case M_JMP:
                     case M_JMP_FALSE:
                     case M_JMP_TRUE:
                        next=jmp_dest(current);
                        if (next!=end && next!=opt_end && *next==M_END_MACRO)
                         { modified++;
                           changed=TRUE;
                           generate_jmp(*current,current,opt_end);
                         }
                     default:
                        current=next_command(current);
                        continue;
                   }
             }
            /*}}}  */
            /*{{{  modifiy some constructs*/
            for (current=start;current!=end;)
             { next=next_command(current);
               switch (*current)
                {
                  case M_CALL:
                     if
                      /*{{{  M_CALL x;M_END_MACRO -> M_JMP x; M_END_MACRO*/
                      (next!=end && *next==M_END_MACRO)
                      { *current=M_JMP;
                        current=next;
                        changed=TRUE;
                        modified++;
                        continue;
                      }
                      /*}}}  */
                     else if
                      /*{{{  M_CALL x; .. x:M_END_MACRO -> O_NOP; O_NOP; ..*/
                      (jmp_dest(current)==opt_end)
                      { *current++=O_NOP;
                        *current++=O_NOP;
                        changed=TRUE;
                        modified++;
                        continue;
                      }
                      /*}}}  */
                     current=next;
                     continue;
                  case M_JMP:
                     dest=jmp_dest(current);
                     if
                      /*{{{  M_JMP x; .. x:M_END_MACRO -> M_END_MACRO; O_NOP; ..*/
                      (dest==opt_end)
                      { *current++=M_END_MACRO;
                        *current++=O_NOP;
                        changed=TRUE;
                        modified++;
                        continue;
                      }
                      /*}}}  */
                     else if
                      /*{{{  M_JMP_X;x: -> O_NOP; O_NOP;*/
                      (dest==next)
                      { *current++=O_NOP;
                        *current++=O_NOP;
                        changed=TRUE;
                        modified++;
                        continue;
                      }
                      /*}}}  */
                     current=next;
                     continue;
                  case M_JMP_FALSE:
                  case M_JMP_TRUE:
                     dest=jmp_dest(current);
                     if
                      /*{{{  M_JMP_COND x;M_END_MACRO;x: -> M_JMP_!COND opt_end;O_NOP;x:*/
                      (next!=end && *next==M_END_MACRO && dest==next_command(next) && opt_end && opt_end!=next)
                      { current=generate_jmp(*current==M_JMP_FALSE?M_JMP_TRUE:M_JMP_FALSE,
                                             current,opt_end);
                        *current++=O_NOP;
                        changed=TRUE;
                        modified++;
                        continue;
                      }
                      /*}}}  */
                     else if
                      /*{{{  M_JMP_COND x;x: -> O_NOP;O_NOP;*/
                      (dest==next)
                      { *current++=O_NOP;
                        *current++=O_NOP;
                        modified++;
                        changed=TRUE;
                        continue;
                      }
                      /*}}}  */
                     current=next;
                     continue;
                  case M_SET_COUNTER:
                     if
                      /*{{{  SET dummy n SUM x dummy -> ADD x n*/
                      (next!=end && *next==M_SUM_COUNTER && *(current+1)==*(next+2)&& isdummy(*(current+1)))
                      { *current++=M_ADD_COUNTER;
                        *current++=*(next+1);
                        current++;
                        *current++=O_NOP;
                        *current++=O_NOP;
                        *current++=O_NOP;
                        modified++;
                        changed=TRUE;
                        continue;
                      }
                      /*}}}  */
                     current=next;
                     continue;
                  case M_ADD_COUNTER:
                     if (end!=next && *next==M_ADD_COUNTER && *(next+1)==*(current+1))
                      /*{{{  ADD x n ADD x m -> ADD x n+m*/
                      { TOKEN *x;

                        for (x=start;x!=end;)
                         /*{{{  search, if command will be twisted by jump*/
                         { switch (*x)
                            { case M_JMP:
                              case M_JMP_FALSE:
                              case M_JMP_TRUE:
                              case M_CALL:
                                 if (jmp_dest(x)==next) break;
                              default:
                                 x=next_command(x);
                                 continue;
                            }
                           break;
                         }
                         /*}}}  */
                        if (x==end)
                         /*{{{  modify it*/
                         { *(next+2)+=*(current+2);
                           *current++=O_NOP;
                           *current++=O_NOP;
                           *current++=O_NOP;
                           modified++;
                           changed=TRUE;
                           continue;
                         }
                         /*}}}  */
                        else
                         /*{{{  leave it*/
                         { current=next;
                           continue;
                         }
                         /*}}}  */
                      }
                      /*}}}  */
                     current=next;
                     continue;
                  default:
                     current=next;
                     continue;
                }
             }
            /*}}}  */
            /*}}}  */
        }
        /*}}}  */
        if (verbose)
         /*{{{  maybe print current optimization strip*/
           if (jmp_cut+modified+shrink)
            { fprintf(stderr,F_OPT_FORMAT,++loop,jmp_cut,modified,shrink);
              jmp_cut=shrink=modified=0;
            }
         /*}}}  */
      }
      /*}}}  */
     while (changed);
     return(end);
   }
#endif
/*}}}  */
/*{{{  parse_term*/
private bool t_unchanged;
private int target;
private int copy_target;
private TOKEN *start_term;
/*{{{  recursion for term-parsing*/
/*{{{  shift_code*/
private TOKEN *shift_code(TOKEN *buff)
{ TOKEN *x;

  if (copy_target>=0) return(buff);
  for (x=buff;x!=start_term;x--,*(x+4)= *x);
  *x++=M_PUSH_INT;
  *x++=(TOKEN)target;
  *x++=M_POP_INT;
  *x++=(TOKEN) (copy_target= -1-copy_target);
  return(buff+4);
}
/*}}}  */

public TOKEN *rpt(tokens token,TOKEN *buff,int dest,int dummy,bool *ret_used)
{ tokens op=name_to_var(token,new_vars_enabled);

  switch (op)
   { case VARIABLE:
      /*{{{  maybe PUSH new POP old*/
      { int x=tk_var->no;

        if (!t_unchanged && x==target)
         { buff=shift_code(buff);x=copy_target; }
        if (x!=dest)
         { *buff++=M_PUSH_INT;*buff++=x;
           *buff++=M_POP_INT;*buff++=dest;
           if (dest==target) t_unchanged=FALSE;
         }
        break;
      }
      /*}}}  */
     case PRE:
      /*{{{  do the action first*/
      { if (!(buff=parse_macro(buff,ret_used))) return(0);
        if (!(buff=rpt(get_token(FALSE),buff,dest,dummy,ret_used))) return(0);
        break;
      }
      /*}}}  */
     case COUNTER:
      /*{{{  convert bool to int*/
      { TOKEN *x;

        if (!(buff=parse_cond(buff,ret_used))) return(0);
        /*{{{  set-counter dummy-0 0*/
        *buff++ = M_SET_COUNTER;
        *buff++ = (TOKEN) dest;
        *buff++ = 0;
        /*}}}  */
        buff=generate_jmp(M_JMP_FALSE,x=buff,buff);
        /*{{{  set-counter dummy-0 1*/
        *buff++ = M_SET_COUNTER;
        *buff++ = (TOKEN) dest;
        *buff++ = 1;
        /*}}}  */
        generate_jmp(M_JMP_FALSE,x,buff);
        break;
      }
      /*}}}  */
     case OPCODE:
       /*{{{  opcode*/
       { *buff++ = M_SET_COUNTER;
         *buff++ = dest;
         *buff++ = tk_key->num;
         break;
       }
       /*}}}  */
     case STORE_CHAR:
     case READ_REPEAT:
     case SET_ENTER:
     case SET_S_ENTER:
     case STORE_X:
     case STORE_Y:
      /*{{{  get data from origami*/
        *buff++ = (token==SET_ENTER)   ? M_ENTERED :
                  (token==STORE_X)     ? M_POS_TO_COUNTER :
                  (token==STORE_Y)     ? M_STORE_LINE_NO :
                  (token==STORE_CHAR)  ? M_STORE_C :
                  (token==READ_REPEAT) ? M_READ_REPEAT :
                                         M_S_ENTERED;
        *buff++ = dest;
        break;
      /*}}}  */
     case ADD:
     case MINUS:
     case MULT:
     case DIV:
     case MOD:
        token=get_token(FALSE);
     case NUMBER:
      {
        /*{{{  if -number, special handling*/
        if (op==NUMBER || (op==MINUS && token==NUMBER))
         /*{{{  M_SET_COUNTER dest value*/
         { *buff++=M_SET_COUNTER;
           *buff++=(TOKEN)dest;
           *buff++=(TOKEN)(op==MINUS?-tk_val:tk_val);
           if (dest==target) t_unchanged=FALSE;
           break;
         }
         /*}}}  */
        if (token!=BEGIN) { error_po();fprintf(stderr,M_WANTBEGIN);return(0); }
        /*}}}  */
        if (!(buff=rpt(get_token(FALSE),buff,dest,dummy,ret_used))) return(0);
        token=name_to_var(get_token(FALSE),new_vars_enabled);
        do
           /*{{{  code one argument from list*/
           switch (token)
            { case COMMA: break;
              /*{{{  VARIABLE - code: op var var*/
              case VARIABLE:
               { int x=tk_var->no;

                 if (!t_unchanged && x==target)
                  { buff=shift_code(buff);x=copy_target; }
                 /*{{{  code the operation*/
                 switch (op)
                  {
                    case MINUS:
                      { *buff++=M_INV_COUNTER;*buff++=x; }
                    case ADD:
                     { *buff++=M_SUM_COUNTER;break; }
                    case MULT:
                     { *buff++=M_MULT;break; }
                    case DIV:
                     { *buff++=M_DIV;break; }
                    case MOD:
                     { *buff++=M_MOD;break; }
                    default:
                     { fprintf(stderr,M_CRASH);return(0); }
                  }
                 /*}}}  */
                 *buff++=(TOKEN)dest;
                 *buff++=x;
                 if (op==MINUS)
                   { *buff++=M_INV_COUNTER;*buff++=x; }
                 if (dest==target) t_unchanged=FALSE;
                 break;
               }

              /*}}}  */
              case NUMBER:
                 if (op==ADD || op==MINUS)
                  /*{{{  code M_ADD_COUNTER dest [-] value*/
                  { *buff++=M_ADD_COUNTER;
                    *buff++=(TOKEN)dest;
                    *buff++=(TOKEN)(op==MINUS?-tk_val:tk_val);
                    if (dest==target) t_unchanged=FALSE;
                    break;
                  }
                  /*}}}  */
              default:
                 if (!(buff=rpt(token,buff,dummy_var(dummy),dummy+1,ret_used)))
                    return(0);
                 /*{{{  code the operation*/
                 switch (op)
                  { case MINUS:
                     { *buff++=M_INV_COUNTER;*buff++=dummy_var(dummy); }
                    case ADD:
                     { *buff++=M_SUM_COUNTER;break; }
                    case MULT:
                     { *buff++=M_MULT;break; }
                    case DIV:
                     { *buff++=M_DIV;break; }
                    case MOD:
                     { *buff++=M_MOD;break; }
                    default:
                     { fprintf(stderr,M_CRASH);return(0); }
                  }
                 /*}}}  */
                 *buff++=dest;
                 *buff++=dummy_var(dummy);
                 if (dest==target) t_unchanged=FALSE;
            }
           /*}}}  */
        while ((token=name_to_var(get_token(FALSE),new_vars_enabled))!=END);
        break;
      }
     /* don't add other case-labels below this line, we need the default
      * as else for MACRO!
      */
     case MACRO:
        if (strlen(tk_string)==1)
         /*{{{  code the integer value of a given character*/
         { *buff++=M_SET_COUNTER;
           *buff++=(TOKEN)dest;
           *buff++=(TOKEN)tk_string[0];
           if (dest==target) t_unchanged=FALSE;
           break;
         }
         /*}}}  */
     default:
      /*{{{  return the error*/
      { error_po();
        fprintf(stderr,M_NOVAR);
        return(0);
      }
      /*}}}  */
   }
  return(buff);
}
/*}}}  */

public TOKEN *parse_term(tokens token,TOKEN *buff,int dest,bool *ret_used)
{ int i;
  static int target_var= -1;

  for (i=DV_MAX;i-->0;) if (dv_list[i]==dest) break;
  t_unchanged=TRUE;
  start_term=buff;
  target=dest;
  if (target_var<0) target_var=creat_var("(target)");
  copy_target= -target_var-1;
  return(rpt(token,buff,dest,i+1,ret_used));
}
/*}}}  */
/*{{{  get_message*/
public TOKEN *get_message(TOKEN *buff)
{
  tokens read=get_token(FALSE);
  TOKEN *x=tk_macro;

  if (read!=MACRO) {
    /*{{{  complex prompt ( ... )*/
    /*{{{  no begin*/
    if (read!=BEGIN) {
      error_po();
      fprintf(stderr,M_NOMESSAGE);
      return(0);
    }
    /*}}}  */
    read=get_token(FALSE);
    while ((read==MACRO || read==COUNTER || read==NAME) && buff)
     { x=tk_macro;
       if (read==MACRO)
        /*{{{  copy the string*/
          while (*x) *buff++ = *x++;
        /*}}}  */
       else if (read==NAME)
        /*{{{  look for msg*/
        { int x=0;

          while (msg_list[x])
             if (strcmp(tk_string,msg_list[x])) x++; else break;
          if (msg_list[x]) *buff++ = -(x+1); else break;
        }
        /*}}}  */
       else
        /*{{{  put var*/
        { *buff++ = M_INT_STRING;
          buff = put_var(buff);
        }
        /*}}}  */
       read=get_token(FALSE);
     }
    /*{{{  errors?*/
    if (!buff) return(0);
    if (read!=END)
     { if (read!=ERROR) { error_po();fprintf(stderr,M_WANTEND); }
       return(0);
     }
    /*}}}  */
    /*}}}  */
  } else
     while (*x) *buff++ = *x++;
  return(buff);
}
/*}}}  */
