/*{{{  #includes*/
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include <local/bool.h>

#define NAME_C

#include "keybind.h"
/*}}}  */

/*{{{  keybinding table type and declaration*/
public KEYNAME bindings[]=
{
  /*{{{  moving cursor*/
  "backward-character",            O_LEFT,
  "forward-character",             O_RIGHT,
  "previous-line",                 O_UP,
  "next-line",                     O_DOWN,
  "beginning-of-line",             O_START_OF_LINE,
  "end-of-line",                   O_END_OF_LINE,
  "previous-page",                 O_PAGE_UP,
  "next-page",                     O_PAGE_DOWN,
  "beginning-of-fold",             O_TOP_OF_FOLD,
  "end-of-fold",                   O_BOT_OF_FOLD,
  "goto-line",                     O_GOTO_LINE,
  /*}}}  */
  /*{{{  folding*/
  "open-fold",                     O_OPEN_FOLD,
  "close-fold",                    O_CLOSE_FOLD,
  "enter-fold",                    O_ENTER_FOLD,
  "exit-fold",                     O_EXIT_FOLD,
  "unfold-fold",                   O_REMOVE_FOLD,
  "create-fold",                   O_CREATE_FOLD,
  "auto-header",                   O_FIRST_LINE_TO_FOLD_HEADER,
  "toggle-file-fold",              O_FILE_FOLD,
  "attach-file-to-fold",           O_ATTACH_FILE,
  /*}}}  */
  /*{{{  editing*/
  "delete-to-end-of-line",         O_DELTO_EOL,
  "delete-line",                   O_DEL_LINE,
  "undo-delete-line",              O_UNDEL_LINE,
  "delete-character",              O_DEL_CHAR_R,
  "delete-previous-character",     O_DELETE,
  "undo-delete-character",         O_UNDEL_CHAR,
  "transpose-characters",          O_TRANSPOSE_CHARACTERS,
  "case-character-toggle",         O_TOGGLE_CASE,
  "newline-and-indent",            O_RETURN,
  "kill-line",                     O_PICK,
  "copy-to-kill-buffer",           O_COPY_PICK,
  "insert-folded-kill-buffer",     O_PUT_PICK,
  "move-line",                     O_MOVE,
  "double-line",                   O_COPY,
  /*}}}  */
  /*{{{  search & replace*/
  "search-forward",                O_FIND,
  "search-reverse",                O_FIND_REVERSE,
  "replace-string",                O_REPLACE,
  "query-replace-string",          O_QUERY_REPLACE,
  "incremental-search-forward",    O_ITS_SEARCH,
  "incremental-search-reverse",    O_ITS_REVERSE,
  /*}}}  */
  /*{{{  files*/
  "read-file",                     O_OPEN_NEW_FILE,
  "save-file",                     O_SAVE_FILE,
  "write-file",                    O_WRITE_FILE,
  "insert-file",                   O_INSERT_FILE,
  "next-file",                     O_NEXT_FILE,
  "previous-file",                 O_PREV_FILE,
  /*}}}  */
  /*{{{  modes and arguments*/
  "set-argument-0",                O_REP_0,
  "set-argument-1",                O_REP_1,
  "set-argument-2",                O_REP_2,
  "set-argument-3",                O_REP_3,
  "set-argument-4",                O_REP_4,
  "set-argument-5",                O_REP_5,
  "set-argument-6",                O_REP_6,
  "set-argument-7",                O_REP_7,
  "set-argument-8",                O_REP_8,
  "set-argument-9",                O_REP_9,
  "add-mode-overwrite",            O_A_OVER,
  "delete-mode-overwrite",         O_D_OVER,
  "add-mode-view",                 O_A_VIEW,
  "delete-mode-view",              O_D_VIEW,
  "add-mode-echo",                 O_A_ECHO,
  "delete-mode-echo",              O_D_ECHO,
  "add-mode-autosave",             O_A_AUTO_SAVE,
  "delete-mode-autosave",          O_D_AUTO_SAVE,
  "numeric-keypad",                O_NUM_ON,
  "application-keypad",            O_APL_ON,
  /*}}}  */
  /*{{{  macros*/
  "define-macro",                  O_DEF_MACRO,
  "execute-macro",                 O_EXE_MACRO,
  "execute-number-macro",          O_CALL_FIX,
  "save-keyboard-macro",           O_DEF_FIX,
  /*}}}  */
  /*{{{  shell commands*/
  "shell",                         O_SHELL,
  "shell-command",                 O_SHELL_COMMAND,
  "suspend-origami",               O_SUSPEND,
  /*}}}  */
  /*{{{  special commands*/
  "status-line-on",                O_TITLE_SHOW,
  "status-line-off",               O_TITLE_HIDE,
  "bell",                          O_BELL,
  "visible-bell",                  O_BELL_VISIBLE,
  "redraw-display",                O_FLUSH,
  "center-and-redraw-display",     O_REFRESH,
  "help",                          O_HELP,
  "describe-bindings",             O_DESCRIBE_BINDINGS,
  "set-language",                  O_LANGUAGE,
  "describe-fold",                 O_FOLD_INFO,
  "set-file-unchanged",            O_UNCHANGE,
  "abort",                         O_BREAK,
  "filter-buffer",                 O_FILTER_BUFFER,
  "pipe-from-command",             O_PIPE_FROM_COMMAND,
  "pipe-kbd-from-command",         O_PIPE_KBD,
  "pipe-to-command",               O_PIPE_TO_COMMAND,
  "quote-character",               K_QUOTE,
  "no-operation",                  O_NOP,
  "shift-next-token",              K_SHIFT,
  /*}}}  */
  /*{{{  how to get out*/
  "save-and-exit-origami",         O_FINISH,
  "exit-origami",                  O_QUIT,
  /*}}}  */
  0,0
};
/*}}}  */
/*{{{  keywords*/
private char file_n_str[tag_length+1]=
                {file_std_c,file_std_c,file_std_c,'F','\0'};
public char end_ref[]="reference-end";
public keywords keytab[]={
  "abort-macro",        DEFAB,
  "add-counter",        ADD_COUNTER,
  "alias",              KEYALIAS,
  "and",                AND,
  "begin-prompt-macro", PRO_IN,
  "case",               CASE,
  "change-fold-type",   CHANGE_FOLD,
  "counter",            COUNTER,
  "counter-0",          COUNTER_NULL,
  "counter>0",          C_POSITIV,
  "default",            DEFAULT,
  "defbinding",         BINDNAME,
  "deffun",             DEFMACRO,
  "defmac",             DEFOP,
  "defmark",            DEFMARK,
  "defset",             DEFSET,
  "defmodes",           MULTIKBD,
  "defmodestring",      DEFMODE,
  "defusermacros",      DEFMYFIX,
  "defvar",             DEFVAR,
  "div",                DIV,
  "do",                 DO,
  "else",               ELSE,
  "end-prompt-macro",   PRO_OUT,
  "esac",               ESAC,
  "execute-fix-macro",  EXMAC,
  "exit",               EXIT,
  "fi",                 FI,
  "forward",            FORWARD,
  "goto",               GOTO_COUNTER,
  "goto-counter",       GOTO_COUNTER,
  "goto-line-counter",  GOTO_Y,
  "if",                 IF,
  "if-using",           COMPIF,
  "in-prompt",          EDITING,
  "initfun",            INITMACRO,
  "inv-counter",        INV_COUNTER,
  "keybind",            KEYDEF,
  "key-not-bound-macro",KNBM,
  "last-message",       LASTMES,
  "local",              LOC,
  "menu-counter",       MENU,
  "message-exit",       MES_EXIT,
  "modulo",             MOD,
  "mode",               KBD,
  "mouse-buttons",      MOUSEMAP,
  "not",                NOT,
  "or",                 OR,
  "pipe-area",          HELP_P,
  "pre",                PRE,
  "prompt",             PROMPT,
  "prompt-counter",     PROMPT_COUNTER,
  "read-newfile-macro", DEFAUTO,
  "read-repeat",        READ_REPEAT,
  "reference-begin",    STARTREF,
  "repeat",             REPEAT,
  "reset-user-mode",    RESET_U_M,
  "return-from-macro",  RETURN,
  "screen-off",         SCROFF,
  "screen-on",          SCRON,
  "save-and-exit-origami-code",
                        EXITCODE,
  "set-counter",        SET_COUNTER,
  "set-cursor-line",    SET_CURSOR,
  "set-enter",          SET_ENTER,
  "set-space-enter",    SET_S_ENTER,
  "set-user-mode",      SET_U_M,
  "show-cursor",        SHOW_CURSOR,
  "store-line",         STORE_Y,
  "store-pos",          STORE_X,
  "sum-counter",        SUM_COUNTER,
  "terminal",           TERM,
  "termalias",          TERMALIAS,
  "test-autosave",      TEST_AUTO,
  "test-begin-fold",    TEST_BEGIN_FOLD,
  "test-begin-line",    TEST_BEGIN_LINE,
  "test-behind-counter",TEST_BEHIND_COUNTER,
  "test-bottom",        TEST_BOTTOM,
  "test-char",          TEST_CHAR,
  "test-char-set",      TEST_CHAR_SET,
  "test-char-less",     TEST_CHAR_LOW,
  "test-char-greater",  TEST_CHAR_HIGH,
  "test-echo",          TEST_ECHO,
  "test-end-fold",      TEST_END_FOLD,
  "test-end-line",      TEST_END_LINE,
  "test-file-changed",  TEST_CHANGED,
  "test-filed",         TEST_FILED,
  "test-fold-line",     TEST_FOLD_LINE,
  "test-language",      TEST_LANG,
  "test-overwrite",     TEST_OVER,
  "test-text",          TEST_TEXT,
  "test-top",           TEST_TOP,
  "test-view",          TEST_VIEW,
  "prompt-char-counter",PROMPT_CHAR,
  "test-char-counter",  TEST_CHAR_C,
  "test-char-less-counter",
                        TEST_CHAR_LOW_C,
  "test-char-greater-counter",
                        TEST_CHAR_HIGH_C,
  "insert-ascii",       INSERT_ASCII,
  "set-area",           HELP_SET,
  "store-character",    STORE_CHAR,
  "use",                COMPUSE,
  "view-macro",         VMAC,
  "while",              WHILE,
  "write-area",         HELP_W,
  file_n_str,           INCLUDE,
  0, ERROR
};
/*}}}  */
