/*{{{  #includes*/
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <local/bool.h>

#define PARSEMAC_C

#include "keybind.h"
/*}}}  */

/*{{{  parse_macro*/
/*{{{  forward opt_parse_mac*/
public TOKEN *opt_parse_macro(TOKEN*,bool*);
/*}}}  */

public TOKEN *parse_macro(TOKEN *buff,bool *ret_used)
{
  tokens token;
  tokens to;

  if (begin_parse()) return(0);
  do {
    token=get_token(FALSE);
    switch (token) {
      /*{{{  repeat*/
      case REPEAT:
       { static int repeat_var= -1;
         bool r_end_used=FALSE;
         TOKEN *call,*test,*ex;

         /*{{{  maybe initialise the repeat-counter!*/
         if (repeat_var==-1) repeat_var=creat_var("(repeat)");
         /*}}}  */
         /*{{{  save old and get current repeat value*/
         *buff++=M_PUSH_INT;
         *buff++=repeat_var;
         if (!(buff=parse_term(get_token(FALSE),buff,repeat_var,ret_used)))
            return(0);
         /*}}}  */
         /*{{{  call the repeated block, prepare the exit-statements*/
         call=buff;
         buff=generate_jmp(M_CALL,call,call);
         *buff++=M_POP_INT;
         *buff++=repeat_var;
         *buff++=M_END_MACRO;
         /*}}}  */
         /*{{{  handle the test*/
         test=buff;
         *buff++=M_POSITIV_COUNTER;
         *buff++=repeat_var;
         ex=buff;
         buff=generate_jmp(M_JMP_FALSE,ex,ex);
         *buff++=M_ADD_COUNTER;
         *buff++=repeat_var;
         *buff++= -1;
         /*}}}  */
         /*{{{  code the block*/
         if (!(buff=parse_macro(buff,&r_end_used))) return(0);
         buff=generate_jmp(M_JMP,buff,test);
         generate_jmp(M_JMP_FALSE,ex,buff);
         /*}}}  */
         if (r_end_used)
          /*{{{  pop the adress for exit-handling*/
          { *ret_used=TRUE;
            generate_jmp(M_CALL,call,test);
            /*{{{  pop the adress (don't execute the M_END_MACRO!)*/
            *buff++=M_POP_INT;
            *buff++=repeat_var;
            /*}}}  */
          }
          /*}}}  */
         else
          /*{{{  call test -> jmp test*/
            generate_jmp(M_JMP,call,test);
          /*}}}  */
         /*{{{  restore old loop counter*/
         *buff++=M_POP_INT;
         *buff++=repeat_var;
         /*}}}  */
         break;
       }
      /*}}}  */
      /*{{{  local*/
      case LOC: {
        /*{{{  comments*/
        /* macros, using return-from-macro must be called via subroutine-call,
         * to guarantee the proper stack-handling. In this case
         *    M_PUSH_INT var1
         *      :
         *    M_PUSH_INT varn
         *    M_CALL x
         *    M_JMP  e
         *  x macro
         *    M_END_MACRO
         *  e M_POP_INT varn
         *      :
         *    M_POP_INT var1
         * is generated instead of
         *    M_PUSH_INT var1
         *      :
         *    M_PUSH_INT varn
         *    macro
         *    M_POP_INT varn
         *      :
         *    M_POP_INT var1
         */
        /*}}}  */

        int vars[var_count];
        int vars_c=0;
        tokens next;

        if (begin_parse()) return(0);
        /*{{{  list of vars*/
        while ((next=name_to_var(get_token(FALSE),new_vars_enabled))!=END) {
          if (next==VARIABLE) {
            /*{{{  already defined var*/
            *buff++ = M_PUSH_INT;
            *buff++ = (TOKEN) (vars[vars_c++]=tk_var->no);
            /*}}}  */
          } else {
            /*{{{  error*/
            if (next!=ERROR) {
              error_po();
              fprintf(stderr,M_NOVAR);
            }
            return(0);
            /*}}}  */
          }
        }
        /*}}}  */
        /*{{{  parse_statement, maybe handle call/jmp*/
        { TOKEN *start=buff,*jump,*block,*end;
          bool ret_used=FALSE;

          jump=buff=generate_jmp(M_CALL,buff,buff);
          block=buff=generate_jmp(M_CALL,buff,buff);
          generate_jmp(M_CALL,start,block);
          if (!(buff=end=parse_macro(buff,&ret_used))) return(0);
          if (ret_used)
           /*{{{  internal macro used return-from-macro -> subroutine-call*/
           { *buff++ =M_END_MACRO;
             generate_jmp(M_JMP,jump,buff);
           }
           /*}}}  */
          else
           /*{{{  normal code, delete call/jmp*/
           { TOKEN *s,*d;

             for (d=start,s=block;s!=end;*(d++)=*(s++));
             buff=d;
           }
           /*}}}  */
        }
        /*}}}  */
        /*{{{  pop all local vars*/
        while (vars_c>0) {
          *buff++ = M_POP_INT;
          *buff++ = vars[--vars_c];
        }
        /*}}}  */
        break;
      }
      /*}}}  */
      /*{{{  while*/
      case WHILE: {
        TOKEN *whilestart,*jmpad;

        if (!(jmpad=parse_cond(whilestart=buff,ret_used))) return(0);
        if (!(buff=generate_jmp(M_JMP_FALSE,jmpad,jmpad))) return(0);
        if (!(buff=parse_macro(buff,ret_used))) return(0);
        if (!(buff=generate_jmp(M_JMP,buff,whilestart))) return(0);
        if (!generate_jmp(M_JMP_FALSE,jmpad,buff)) return(0);
        break;
      }
      /*}}}  */
      /*{{{  do*/
      case DO: {
        TOKEN *dostart=buff;

        if (!(buff=parse_macro(buff,ret_used))) return(0);
        /*{{{  read the while*/
        if ((to=get_token(FALSE))!=WHILE) {
          if (to!=ERROR) {
            error_po();
            fprintf(stderr,M_NOWHILE);
          }
          return(0);
        }
        /*}}}  */
        if (!(buff=parse_cond(buff,ret_used))) return(0);
        if (!(buff=generate_jmp(M_JMP_TRUE,buff,dostart))) return(0);
        break;
      }
      /*}}}  */
      /*{{{  case*/
      case CASE: {
        TOKEN *defaultjmp=0,*condptr;

        token=get_token(FALSE);
        do {
          switch (token) {
            /*{{{  BEGIN -> cond macro*/
            case BEGIN: {
              if (!(condptr=parse_cond(buff,ret_used))) return(0);
              if (!(buff=generate_jmp(M_JMP_FALSE,condptr,condptr))) return(0);
              if (!(buff=parse_macro(buff,ret_used))) return(0);
              if (defaultjmp) {
                if (!(buff=generate_jmp(M_JMP,buff,defaultjmp))) return(0);
              } else
                buff=generate_jmp(M_JMP,defaultjmp=buff,buff);
              if (!generate_jmp(M_JMP_FALSE,condptr,buff)) return(0);
              if (end_parse(M_CASEEND)) return(0);
              token=get_token(FALSE);
              break;
            }
            /*}}}  */
            /*{{{  DEFAULT*/
            case DEFAULT:
              if (!(buff=parse_macro(buff,ret_used))) return(0);
              token=get_token(FALSE);
              break;
            /*}}}  */
            case ESAC:
              break;
            default:
              if (token!=ERROR) {
                error_po();
                fprintf(stderr,M_NOESAC);
              }
              return(0);
          }
        } while (token!=ESAC);
        if (defaultjmp && !generate_jmp(M_JMP,defaultjmp,buff)) return(0);
        break;
      }
      /*}}}  */
      /*{{{  if*/
      case IF: {
        TOKEN *jmpadr,*endtrue;

        if (!(jmpadr=parse_cond(buff,ret_used))) return(0);
        if (!(buff=generate_jmp(M_JMP_FALSE,jmpadr,jmpadr))) return(0);
        if (!(endtrue=parse_macro(buff,ret_used))) return(0);
        token=get_token(FALSE);
        switch (token) {
          /*{{{  if else fi*/
          case ELSE:
            if (!(buff=generate_jmp(M_JMP,endtrue,endtrue))) return(0);
            if (!generate_jmp(M_JMP_FALSE,jmpadr,buff)) return(0);
            if (!(buff=parse_macro(buff,ret_used))) return(0);
            if (!generate_jmp(M_JMP,endtrue,buff)) return(0);
            if ((to=get_token(FALSE))!=FI) {
              if (to!=ERROR) {
                error_po();
                fprintf(stderr,M_NOFI);
              }
              return(0);
            }
            break;
          /*}}}  */
          /*{{{  if fi*/
          case FI:
            if (!generate_jmp(M_JMP_FALSE,jmpadr,endtrue)) return(0);
            buff=endtrue;
            break;
          /*}}}  */
          default:
            if (token!=ERROR) {
              error_po();
              fprintf(stderr,M_NOELSEFI);
            }
            return(0);
        }
        break;
      }
      /*}}}  */
      /*{{{  return-from-macro*/
      case RETURN:
        *ret_used=TRUE;
        *buff++ = M_END_MACRO;
        break;
      /*}}}  */
      /*{{{  simple commands, which are not allowed on keyboard!*/
      case SCROFF: *buff++=M_SCREEN_OFF;break;
      case SCRON: *buff++=M_SCREEN_ON;break;
      /*}}}  */
      /*{{{  set/reset-user-mode*/
      case SET_U_M:
      case RESET_U_M: {
        int i=used_modes-1;

        *buff++ = (token==SET_U_M) ? M_SET_USER_MODE:
                                     M_RESET_USER_MODE;
        if ((token=get_token(FALSE))!=NAME) i= -1;
        while (i>=0) if (!strcmp(tk_string,modenames[i])) break; else i--;
        if (i<0)
         { error_po();fprintf(stderr,M_UMNAME);return(0); }
        *buff++ = (TOKEN) i;
        break;
      }
      /*}}}  */
      /*{{{  set-counter*/
      case SET_COUNTER:
       { if (!put_var(buff)) return(0);
         if (!(buff=parse_term(get_token(FALSE),buff,*buff,ret_used))) return(0);
         break;
       }
      /*}}}  */
#      ifdef OLD_OCL
      /*{{{  commands, only for historical reasons*/
      /*{{{  sum-counter*/
      case SUM_COUNTER:
        error_po();fprintf(stderr,"old-fashioned command, will be removed soon%s",verbose?"\n\n\n":"\n");
        *buff++ = M_SUM_COUNTER;
        if (!(buff=put_var(buff))) return(0);
        if (!(buff=put_var(buff))) return(0);
        break;
      /*}}}  */
      /*{{{  inv-counter + get data from origami*/
      case STORE_CHAR:
      case READ_REPEAT:
      case INV_COUNTER:
      case SET_ENTER:
      case SET_S_ENTER:
      case STORE_X:
      case STORE_Y:
        error_po();fprintf(stderr,"old-fashioned command, will be removed soon%s",verbose?"\n\n\n":"\n");
        *buff++ = (token==SET_ENTER)   ? M_ENTERED :
                  (token==INV_COUNTER) ? M_INV_COUNTER :
                  (token==STORE_X)     ? M_POS_TO_COUNTER :
                  (token==STORE_Y)     ? M_STORE_LINE_NO :
                  (token==STORE_CHAR)  ? M_STORE_C :
                  (token==READ_REPEAT) ? M_READ_REPEAT :
                                         M_S_ENTERED;
        if (!(buff=put_var(buff))) return(0);
        break;
      /*}}}  */
      /*{{{  add-counter*/
      case ADD_COUNTER:
       { int inv=1;

         error_po();fprintf(stderr,"old-fashioned command, will be removed soon%s",verbose?"\n\n\n":"\n");
         *buff++ = (token==SET_COUNTER) ? M_SET_COUNTER :
                                          M_ADD_COUNTER;
         if (!(buff=put_var(buff))) return(0);
         while ((to=get_token(FALSE))==MINUS) inv = -inv;
         /*{{{  if no number given, error*/
         if (to!=NUMBER) {
           if (to!=ERROR) {
             error_po();
             fprintf(stderr,M_NOADDCOUNTER);
           }
           return(0);
         }
         /*}}}  */
         *buff++ = (TOKEN) (tk_val*inv);
         break;
       }
      /*}}}  */
      /*}}}  */
#      endif
      /*{{{  commands with one term argument*/
      case SET_CURSOR:
      case SHOW_CURSOR:
      case COUNTER:
      case HELP_SET:
      case INSERT_ASCII:
      case GOTO_COUNTER:
      case GOTO_Y:
       { bool d=FALSE;

         to=name_to_var(get_token(FALSE),new_vars_enabled);
         if (to!=VARIABLE)
          { if (!(buff=parse_term(to,buff,dummy_var(0),ret_used))) return(0);
            d=TRUE;
          }
         if (token==GOTO_Y)
          { *buff++ = O_GOTO_LINE;*buff++ = M_INT_STRING; }
         else
          { *buff++ = (token==GOTO_COUNTER)? M_GO_COUNTER_X_POS :
                      (token==INSERT_ASCII)? M_ASCII :
                      (token==HELP_SET)    ? M_SHOW_HELP :
                      (token==SHOW_CURSOR) ? M_SHOW_CURSOR :
                      (token==SET_CURSOR)  ? O_FLUSH :
                                             M_INT_STRING;
          }
         *buff++=(TOKEN)(d?dummy_var(0):tk_var->no);
         if (token==GOTO_Y) *buff++ = O_RETURN;
         break;
       }
      /*}}}  */
      /*{{{  commands with two term argument*/
      case CHANGE_FOLD:
       { int n1;
         int n2;

         /*{{{  get term 1*/
         to=name_to_var(get_token(FALSE),new_vars_enabled);
         if (to!=VARIABLE)
          { if (!(buff=parse_term(to,buff,dummy_var(0),ret_used))) return(0);
            n1=dummy_var(0);
          }
         else
            n1=tk_var->no;
         /*}}}  */
         /*{{{  get term 2*/
         to=name_to_var(get_token(FALSE),new_vars_enabled);
         if (to!=VARIABLE)
          { if (!(buff=parse_term(to,buff,dummy_var(0),ret_used))) return(0);
            n2=dummy_var(0);
          }
         else
            n2=tk_var->no;
         /*}}}  */
         *buff++ = M_CHANGE_FOLD;
         *buff++=(TOKEN)n1;
         *buff++=(TOKEN)n2;
         break;
       }
      /*}}}  */
      /*{{{  commands with prompts*/
      case EXITCODE:
      case MENU:
      case HELP_W:
      case HELP_P:
      case PROMPT:
      case PROMPT_CHAR:
      case PROMPT_COUNTER: {
        /*{{{  generate base of the command*/
        if (token==EXITCODE)
         /*{{{  exit with code and prompt*/
         { *buff++ = M_FINISH;
           if (!(buff=put_var(buff))) return(0);
         }
         /*}}}  */
        else if (token==PROMPT_CHAR)
         /*{{{  promptc*/
         { *buff++ = M_PROMPT_C;
           if (!(buff=put_var(buff))) return(0);
         }
         /*}}}  */
        else if (token==HELP_P)
         /*{{{  pipehelp*/
         { *buff++ = M_PIPE_HELP; }
         /*}}}  */
        else if (token==HELP_W)
         /*{{{  pipe-help var var*/
         { *buff++ = M_WRITE_HELP;
           if (!(buff=put_var(buff))) return(0);
           if (!(buff=put_var(buff))) return(0);
         }
         /*}}}  */
        else if (token==PROMPT_COUNTER)
         /*{{{  prompt var*/
         { *buff++ = M_PROMPT;
           if (!(buff=put_var(buff))) return(0);
         }
         /*}}}  */
        else if (token==MENU)
         /*{{{  prompt var*/
         { *buff++ = M_MENU;
           if (!(buff=put_var(buff))) return(0);
         }
         /*}}}  */
        else
         /*{{{  prompt -1*/
         { *buff++ = M_PROMPT;
           *buff++ = -1;
         }
         /*}}}  */
        /*}}}  */
        /*{{{  prompt*/
        if (!(buff=get_message(buff))) {
          fprintf(stderr,M_NOPROMPT);
          return(0);
        }
        /*}}}  */
        *buff++ = M_END_MACRO;
        break;
      }
      /*}}}  */
      /*{{{  message-exit/exit*/
      case MES_EXIT:
      case EXIT: {
         *buff++ = M_EXIT;
         if (token==MES_EXIT && !(buff=get_message(buff))) {
           fprintf(stderr,M_NOMESSAGE);
           return(0);
         }
         *buff++ = M_END_MACRO;
         break;
      }
      /*}}}  */
      /*{{{  opcode*/
      case OPCODE:
        if (tk_key->num!=O_NOP) *buff++=tk_key->num;
        if (tk_key->num==O_FLUSH) *buff++=cl;
        break;
      /*}}}  */
      /*{{{  macrostring*/
      case MACRO: {
        TOKEN *x=tk_macro;

        while (*x) *buff++ = *x++;
        break;
      }
      /*}}}  */
      /*{{{  previosly defined operation*/
      case OPERATION: {
        /*{{{  comments*/
        /* macros, using return-from-macro must be called via subroutine-call,
         * to guarantee  the  proper  semantics.  The  return should abort the
         * called macro and not the current one. In this case
         *    M_CALL x
         *    M_JMP  e
         *  x macro
         *    M_END_MACRO
         *  e
         * is generated.
         */
        /*}}}  */

        TOKEN *x=tk_operation->ops;
        int lg=tk_operation->length;
        TOKEN *call_ad,*jmp_ad=0;

        /*{{{  maybe subroutine handling*/
        if (tk_operation->ret_used)
         { call_ad=buff;
           jmp_ad=buff=generate_jmp(M_CALL,buff,buff);
           buff=generate_jmp(M_JMP,buff,buff);
           generate_jmp(M_CALL,call_ad,buff);
         }
        /*}}}  */
        /*{{{  copy the macro*/
        while (lg--) *buff++ = *x++;
        /*}}}  */
        /*{{{  maybe subroutine handling*/
        if (tk_operation->ret_used)
         { *buff++ = M_END_MACRO;generate_jmp(M_JMP,jmp_ad,buff); }
        /*}}}  */
        break;
      }
      /*}}}  */
      /*{{{  automacro/abortmacro/promptmacros/knb/view*/
      case VMAC:
      case KNBM:
      case PRO_IN:
      case PRO_OUT:
      case DEFAB:
      case DEFAUTO: {
        int x= token==DEFAB?ab_macro:
              (token==PRO_IN?proin:
              (token==PRO_OUT?proout:
              (token==KNBM?knbm_no:
              (token==VMAC?vm_no:
                      auto_macro))));

        if (!x)
         { error_po();
           fprintf(stderr,M_NOMSTRING);
           return(0);
         }
        *buff++=O_EXE_MACRO+x;
        break;
      }
      /*}}}  */
      case END: break;
      /*{{{  rekursiv on my own*/
      case NAME:
        if (!strcmp(op_def_name,tk_string)) {
          buff=generate_jmp(M_CALL,buff,m_def);
          break;
        }
      /*}}}  */
      /*{{{  default=error*/
      default:
        if (token!=ERROR) {
          error_po();
          fprintf(stderr,M_NOMSTRING);
        }
        return(0);
      /*}}}  */
    }
  }
  while (token!=END);
  return(buff);
}
/*}}}  */
/*{{{  opt_parse_macro*/
public TOKEN *opt_parse_macro(TOKEN *buff,bool *ret_used)
{ TOKEN *x;

  x=parse_macro(buff,ret_used);
  if (!x) return(0);
  if (do_opt) x=opt_mac(buff,x,FALSE);
  return(x);
}
/*}}}  */
