/*{{{  possible defines*/
/*
   Use the following configuration symbols to generate a makefile for
   your system with "/usr/lib/cpp -DCONFIG Makefile.cpp >Makefile"

     SUNOS_GCC
     HPUX_ANSI
     LINUX
   Set the variables for unproto to your version of it.
     SUNOS_UNPROTO_CC
     HPUX_UNPROTO_KR
   The following defines are not tested. This version must be compiled with
   ANSI-cc or unproto!
     MINIX_BCCi386  (untested)
     MINIX_GCCi386  (untested)
     MIPS           (untested)

   The file config.h will be created from the Makefile. The  following  defines
   for config.h are supported:

   VARARGS      if your system is screwed up with varargs
   NO_MSG       to avoid the copyright message startup - for personal use!
   VIRTUAL      use code to store lines of text in a file
   NOSYSEXIT    -DNOSYSEXIT enables the lokal definition of the standard unix
                exit codes. All following standard exit-codes can be  redefined
                with -DSYMBOL=xx too:
                   EX_OK            default is  0
                   EX_USAGE         default is 64
                   EX_DATAERR       default is 65
                   EX_NOINPUT       default is 66
                   EX_UNAVAILABLE   default is 69
                   EX_SOFTWARE      default is 70
                   EX_OSERR         default is 71
                   EX_CANTCREAT     default is 73
                 If there are no -DSYMBOL=xx definitons,  the  above  mentioned
                 defaults are used.
   ORIEXITBASE  -DORIEXITBASE=xx sets the startvalue for the non-standard exit
                codes to xx (default is 1). Origami uses the values  xx,xx+1,..
                for its non-standard exits.
   NOMEMDUMP    -DNOMEMDUMP=xx try to dump current file if memory full
		xx is the size of a malloced buffer, freed before dumping!
		(needs sep. stack & heap machines)
   LINELEN      -DLINELEN=xx defines maximal line length (default 256)
   M_DEPTH      -DM_DEPTH=xx defines standard stackdepth for macro execution (default 64)
   HOMERC       enables origami to search rc-file in $HOME
   ORIGAMI_PATH -DORIGAMI_PATH=\"xx\" enables Origami to search for rc-file in xx
   NO_MAIL_S    -DMAIL_NO_S supresses the usage of -s for a oricore-mail
   MAILCMD      -DMAILCMD(c,s,u)=xxx replaces the default mail commandstring
                with xxx.
                   c is the buffer for the string
                   s is the subject
                   u is the user
   MOUSY        use code for mouse preparation (see doc/MOUSY!), set by XTERM
                and MGR
   DEFAULT_MOUSE
                -DDEFAULT_MOUSE=xxx sets the mouse-support, wich  is  activated
                in keybind without any -M.. option. The default is:
                       xterm, if possible
                  else mgr,   if possible
                  else ""
   MGR          enable the support for mgr-mouse
   XTERM        enable the support for xterm-mouse-usage
		Xterm is send the escape-sequence, to send mouse-position
		on 'button-pressed'.
    XTERM_START_BP -DXTERM_START_BP=xx
		  sets the string, to activate sending the position, default is
		  "\033[?9h"
    XTERM_END_BP -DXTERM_END_BP=xx
		  sets the string, to deactivate sending the position, default
		  is "\033[?9l".
    XTERM_MOUSE_CODE
                  -DXTERM_MOUSE_CODE=xx sets the  answering  header,  which  is
                  followed by button,x and y-position, default is "\033[M".
    XTERM_MOUSE_LG
                  -DXTERM_MOUSE_LG=xx  sets  the  length  of  XTERM_MOUSE_CODE,
                  default is 3.
    XTERM_CHGNAME sets the c-source to change the windowname. Default is
                  sending esc ] 0 ; <name> bel to stdout.
    XTERM_DEFAULT_TITLE
                  specifies the name, origami set the window tile on exiting,
                  if -T isused without another name

   The following switches can be used to compile Origami on systems, where some
   library functions are not supported:

   USLEEP_MIS   Set this, if your system has no usleep command. In this case the file
                usleep.$O must be in the ADD_FILES definition! Choose on of the
                following simulations:
    SELECT_USLEEP   usleep(x)->sleep(x/1000000);select(dummy,x%1000000);
                      (tested with SUNOS_GCC,SUNOS_UNPROTO_CC,HPUX_ANSI and HPUX_UNPROTO)
    POLL_USLEEP     usleep(x)->sleep(x/1000000);poll(dummy,x*1000);
                      (tested with SUNOS_GCC and SUNOS_UNPROTO_CC)
    BUSY_USLEEP     usleep(x)->sleep(x/1000000);busy-wait(x%1000000); (using gettimeofday!)
                      (tested with SUNOS_GCC,SUNOS_UNPROTO_CC,HPUX_ANSI and HPUX_UNPROTO)
                If no of the above is used, usleep is replaced in the following
                way:
                    usleep(x)->sleep(x/1000000);if (x%100000) sleep(1);
   STRERROR_MIS Set this, if you system has no strerror command. The file strerror.$O
                must be in the ADD_FILES definition
   STRRSTR_MIS  Set this, if your system has no strrstr command. The file strrstr.$O
                must be in the ADD_FILES definition
   PUTENV_MIS   Set this, if your system has no putenvcommand. The file putenv.$O
                must be in the ADD_FILES definition
*/
/*}}}  */
/*{{{  SUNOS*/
/*{{{  UNPROTO_CC*/
#ifdef SUNOS_UNPROTO_CC
AWK=            awk
CPP_DIR=        /home/balu/ourbin/bin.$(ARCHITECTURE)
CONFIG=         config.h
CFLAGS=         -O4 -Qpath $(CPP_DIR) -I.. -DCONFIG_H
LDFLAGS=
LIBS=           -ltermcap
O=              o
ADD_FILES=	strerror.$O strrstr.$O
#ifndef RC_PATH
RC_PATH=        $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif

default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define VARARGS"                          >>config.h
		echo "#define STRERROR_MIS"                     >>config.h
		echo "#define STRRSTR_MIS"                      >>config.h
		echo "#define XTERM"                            >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define LINELEN 1024"                     >>config.h
#endif
/*}}}  */
/*{{{  GCC*/
#ifdef SUNOS_GCC
AWK=            awk
CC=             gcc


CONFIG=         config.h
CFLAGS=         -O -I.. -ansi -pedantic -Wall -DCONFIG_H
LDFLAGS=
LIBS=           -ltermcap
O=              o
ADD_FILES=	strerror.$O strrstr.$O
#ifndef RC_PATH
RC_PATH=	$(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=	$(HOME)/bin$(ARCHITECTURE)
#endif

default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define _POSIX_PATH_MAX 255"              >>config.h
		echo "#define STRERROR_MIS"                     >>config.h
		echo "#define STRRSTR_MIS"                      >>config.h
		echo "#define XTERM"                            >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define LINELEN 1024"                     >>config.h
		echo "#define " `mach`                          >>config.h
                     /* this may faile: sun include file "sys/wait.h" has NO */
                     /* defaults! maybe use another  machine-typ  or  delete */
                     /* -ansi -pedantic                                      */
#endif
/*}}}  */
/*}}}  */
/*{{{  HPUX*/
/*{{{  HPUX_UNPROTO*/
#ifdef HPUX_UNPROTO
CPP=/home/balu/ourbin/bin.hp/cpp
CC=             cc
AWK=            awk
CONFIG=         config.h
CFLAGS=         -O -tp,$(CPP) -I.. -DCONFIG_H
LDFLAGS=
LIBS=           -ltermcap
O=              o
ADD_FILES=	usleep.$O
#ifndef RC_PATH
RC_PATH=        $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif

default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define VARARGS"                          >>config.h
		echo "#define USLEEP_MIS"                       >>config.h
		echo "#define SELECT_USLEEP"                    >>config.h
		echo "#ifdef USLEEP_C"                          >>config.h
		echo "#define _INCLUDE_HPUX_SOURCE"             >>config.h
		echo "#endif"                                   >>config.h
		echo "#define MAIL_NO_S"                        >>config.h
		echo "#define XTERM"                            >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define LINELEN 1024"                     >>config.h
#endif
/*}}}  */
/*{{{  HPUX_ANSI*/
#ifdef HPUX_ANSI
CC=             cc
AWK=            awk
CONFIG=         config.h
CFLAGS=         -Aa +O3 -I.. -DCONFIG_H
LDFLAGS=
LIBS=           -ltermcap
O=              o
ADD_FILES=	usleep.$O
#ifndef RC_PATH
RC_PATH=        $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin$(ARCHITECTURE)
#endif

default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define USLEEP_MIS"                       >>config.h
		echo "#define SELECT_USLEEP"                    >>config.h
		echo "#ifdef USLEEP_C"                          >>config.h
		echo "#define _INCLUDE_HPUX_SOURCE"             >>config.h
		echo "#endif"                                   >>config.h
		echo "#define MAIL_NO_S"                        >>config.h
		echo "#define XTERM"                            >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define LINELEN 1024"                     >>config.h
#endif
/*}}}  */
/*}}}  */
/*{{{  LINUX*/
#ifdef LINUX
AWK=            gawk
CC=             gcc
CONFIG=         config.h
CFLAGS=         -Wall -DUSG -I.. -DCONFIG_H
LDFLAGS=
LIBS=
O=              o
/* small hack, disappears in next lib release */
ADDFILES=	strrstr.$O putenv.$O usleep.$O
#ifndef RC_PATH
RC_PATH=	/usr/local/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       /usr/local/bin
#endif

default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define STRERROR_MIS"                     >>config.h
		echo "#define STRRSTR_MIS"                      >>config.h
		echo "#define PUTENV_MIS"                       >>config.h
		echo "#define USLEEP_MIS"                       >>config.h
		echo "#define NOSYSEXIT"                        >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NO_MSG"                           >>config.h
#endif
/*}}}  */
/*{{{  MINIX*/
/*{{{  BCCi386*/
#ifdef MINIX_BCCi386
AWK=            bawk
CC=             bcc -3
CONFIG=         config.h
CFLAGS=         -O -DCONFIG_H
LDFLAGS=        -s
LIBS=
O=              o
ADD_FILES=	strrstr.$O usleep.$O
#ifndef RC_PATH
RC_PATH=        /usr/local/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       /usr/local/bin
#endif

default: origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define VIRTUAL"                          >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define _MINIX"                           >>config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NOSYSEXIT"                        >>config.h
		echo "#define STRRSTR_MIS"                      >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
#endif
/*}}}  */
/*{{{  GCCi386*/
#ifdef MINIX_GCCi386
AWK=            bawk
CC=             gcc
CONFIG=         config.h
CFLAGS=         -O -ansi -pedantic -Wall -DCONFIG_H
LDFLAGS=        -s
LIBS=
O=              o
ADD_FILES=	strrstr.$O usleep.$O
#ifndef RC_PATH
RC_PATH=   /usr/local/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       /usr/local/bin
#endif

default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define VIRTUAL"                          >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define _MINIX"                           >>config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define NOSYSEXIT"                        >>config.h
		echo "#define STRRSTR_MIS"                      >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
#endif
/*}}}  */
/*}}}  */
/*{{{  MIPS*/
#ifdef MIPS
CC=		/posix/bin/cc
AWK=            awk
CONFIG=         config.h
CFLAGS=         -O -I.. -DCONFIG_H
LDFLAGS=        -s
LIBS=           /usr/posix/usr/lib/libc.a -L/usr/lib -ltermcap
O=              o
ADD_FILES=	strerror.$O strrstr.$O usleep.$O
#ifndef RC_PATH
RC_PATH=        $(HOME)/origami
#endif
#ifndef BIN_PATH
BIN_PATH=       $(HOME)/bin
#endif


default:	origami

config.h:
		echo "/*configuration, generated by make:*/"     >config.h
		echo "#define LINELEN 1024"                     >>config.h
		echo "#define NO_MSG"                           >>config.h
		echo "#define _POSIX_SOURCE"                    >>config.h
		echo "#define HOMERC"                           >>config.h
		echo "#define STRRSTR_MIS"                      >>config.h
		echo "#define STRERROR_MIS"                     >>config.h
		echo "#define ORIGAMI_PATH \"$(RC_PATH)\""      >>config.h
		echo "#define XTERM"                            >>config.h
#endif
/*}}}  */

HDRS=		exit.h getmsg.h macros.h origami.h ../h/keys.h ../h/os.h \
		../h/token.h $(CONFIG)

OBJS=		display.$O fieldedit.$O finds.$O foldfiling.$O foldhelp.$O \
		folding.$O getmsg.$O gettoken.$O init.$O keyboard.$O keytab.$O \
		loop.$O main.$O messages.$O misc.$O oriedt.$O prompts.$O \
		readfolds.$O screen.$O set.$O shell.$O signals.$O string.$O \
		virtual.$O writefolds.$O $(ADD_FILES)

origami:        $(OBJS)
		$(CC) $(LDFLAGS) -o $@ $(OBJS) $(LIBS)

/*{{{  internal compile rules*/
$(OBJS):        $(HDRS)

init.$O set.$O:	../h/rcformat.h

init.$O keyboard.$O misc.$O screen.$O shell.$O:	../h/envvar_str.h

keytab.$O:	permlist.h

permlist.h:     ../h/tokens tok2h.awk
		$(AWK) -f tok2h.awk ../h/tokens >permlist.h

../h/token.h:   ../h/tokens ../h/tok2h.awk
		$(AWK) -f ../h/tok2h.awk ../h/tokens >../h/token.h

getmsg.h:       msg2h.awk msg
		$(AWK) -F@ -f msg2h.awk msg >getmsg.h

/*}}}  */

install:        codeinstall msginstall

/*{{{  internal install rules*/
.origamimsg:    msg2dot.awk msg
		$(AWK) -f msg2dot.awk msg >.origamimsg

msginstall:	.origamimsg
		install -d $(RC_PATH)
		install -m 644 .origamimsg $(RC_PATH)

codeinstall:	origami
		install -d $(BIN_PATH)
		install -s origami $(BIN_PATH)
		strip $(BIN_PATH)/origami

deinstall:
		rm -f $(BIN_PATH)/origami $(RC_PATH)/.origamimsg
/*}}}  */

clean:
		rm -f core symbol.out err.out $(OBJS) getmsg.h permlist.h

clobber:        clean
		rm -f origami Makefile .origami* ../h/token.h permlist.h $(CONFIG)
