/*{{{  #includes*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <local/bool.h>

#include "pager.h"
#include "../h/envvar_str.h"
/*}}}  */

/*{{{  strings*/
char *title=       TITEL;
char ORIAREA[]=    AREAVAR;
char DEFAULTF[]=   "default";
char ORIDIRCHGF[1024];
char ERRFNF[1024];
char ERRFOR[1024];
#define NO_FORM    "[[[[[(((((no-format)))))]]]]]"
/*}}}  */
/*{{{  types*/
typedef struct
 { char *name;
   char *form;
 } F_L;
/*}}}  */
/*{{{  variables*/
/*{{{  prototypes for getopt*/
int getopt(int,char**,char*);
extern char *optarg;
extern int optind;
/*}}}  */
int blocklength=22;
int blockbegin=1;
int blockoffset=0;
bool lead=FALSE;
bool usage=FALSE;
bool std=FALSE;
bool useline=FALSE;
char *varname="";
char *FORVAR="";
char formatname[1024]=NO_FORM;
char os_std_format[128]=OS_F_STR;
F_L f_list[]=
#   ifdef OS_F_LIST
      OS_F_LIST
#   else
      /*{{{  default-list*/
      { "sun-gcc",   "%[^:]:%d:",
        "sun-cc",    "\"%[^\"]\", line %d:,",
        "keybind",   "file: %s line: %d",
        DEFAULTF,    os_std_format,
        "",          os_std_format
      }
      /*}}}  */
#   endif
   ;
/*}}}  */

/*{{{  do_usage*/
void do_usage(void)
{
  printf("usage: oripager [-hmsL] [-b blocknumber] [-l blocklength] [-o offsett] [-t title] [-v environment-variable] [-f format-name] [-F format-var] [filename]");
  exit(0);
}
/*}}}  */

/*{{{  main*/
void main(int argc,char **argv)
{
  /*{{{  variables*/
  FILE *in;
  int startline=1;
  int line=1;
  char str[4096];
  char str1[4096];
  char dir[4096];
  char titlestr[256];
  char *titlefile=titlestr+strlen(NOFILE);
  bool off=FALSE;
  bool eof_met=FALSE;
  /*}}}  */

  /*{{{  init strings*/
  strcpy(titlestr,NOFILE);
  strcpy(ORIDIRCHGF,ORIPAGERDIR);
  strcat(ORIDIRCHGF," \"%[abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,-_=+\\|/@#$%^!*&]\"");
  strcpy(ERRFNF,OP_E_FORMATNAME);
  strcat(ERRFNF," \"%[abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,-_=+\\|/@#$%^!*&]\"");
  strcpy(ERRFOR,OP_E_FORMATSTRING);
  strcat(ERRFOR," \"");
  /*}}}  */
  /*{{{  parse arguments*/
  { int c;

    while ((c=getopt(argc,argv,"b:f:hl:mo:st:v:F:L"))!=EOF)
    switch (c)
     {
       /*{{{  b*/
       case 'b': blockbegin=atoi(optarg);break;
       /*}}}  */
       /*{{{  f*/
       case 'f': strcpy(formatname,optarg);break;
       /*}}}  */
       /*{{{  l*/
       case 'l': blocklength=atoi(optarg);break;
       /*}}}  */
       /*{{{  m*/
       case 'm': lead=TRUE;break;
       /*}}}  */
       /*{{{  o*/
       case 'o': blockoffset=atoi(optarg);break;
       /*}}}  */
       /*{{{  s*/
       case 's': std=TRUE;break;
       /*}}}  */
       /*{{{  t*/
       case 't': title=optarg;break;
       /*}}}  */
       /*{{{  v*/
       case 'v': varname=optarg;break;
       /*}}}  */
       /*{{{  F*/
       case 'F': FORVAR=optarg;break;
       /*}}}  */
       /*{{{  L*/
       case 'L': useline=TRUE;break;
       /*}}}  */
       case 'h':
       default:
          do_usage();
     }
    if ((std&&argv[optind])||(!argv[optind]||argv[optind+1])) do_usage();
  }
  /*}}}  */
  /*{{{  F.. given, load the needed format*/
  if (*FORVAR)
   { char *x=getenv(FORVAR);

     if (x)
      { strcpy(os_std_format,x);
        strcpy(formatname,DEFAULTF);
      }
   }
  /*}}}  */
  /*{{{  set in to correct file*/
  *dir='\0';
  if (std)
   /*{{{  use stdin*/
   { in=stdin;
     strcpy(titlefile,STDINNAME);
   }
   /*}}}  */
  else if (argv[optind])
   /*{{{  use given file*/
   { in=fopen(argv[1],"r");
     strcpy(titlefile,argv[1]);
   }
   /*}}}  */
  else
   /*{{{  try to open standard files*/
   { char *oriarea=getenv(*varname?varname:ORIAREA);

     if (!oriarea)
      { strcpy(titlestr,NOVAR);
        strcat(titlestr,*varname?varname:ORIAREA);
        in=0;
      }
     else
      { strcpy(titlefile,oriarea);in=fopen(oriarea,"r"); }
   }
   /*}}}  */
  /*}}}  */
  /*{{{  initialise number of needed line*/
  if (blockbegin<=0) blockbegin=1;
  if (blocklength<=0) blocklength=22;
  off=blockoffset;
  if (blockoffset<=0 || blockoffset>blocklength)
     blockoffset=1;
  if (useline)
   { startline=blockbegin; }
  else
   /*{{{  get line for block*/
   {
     /*{{{  combine blocklength and title*/
     if (*title)
        if (blocklength==1)
           *title='\0';
        else
           blocklength--;
     /*}}}  */
     startline=(blockbegin-1)*blocklength+blockoffset;
     /*{{{  with offset -> only one line without header*/
     if (off)
      { *title='\0';
        blocklength=1;
      }
     /*}}}  */
   }
   /*}}}  */
  /*}}}  */
  /*{{{  overread starting lines*/
  while (line!=startline)
     if (!in || !fgets(str,4096,in))
        break;
     else
      { line++;
        while ((str[strlen(str)-1]=='\n') || (str[strlen(str)-1]=='\r'))
           str[strlen(str)-1]='\0';
        if (strcmp(formatname,NO_FORM))
         { if (sscanf(str,ORIDIRCHGF,str1)==1)
            { strcpy(dir,str1); printf("dir %s\n",dir);}
           if (sscanf(str,ERRFNF,str1)==1)
            { strcpy(formatname,str1); printf("fname %s\n",formatname);}
           if (   !strncmp(str,ERRFOR,strlen(ERRFOR))
               && strlen(str)>strlen(ERRFOR)
               && str[strlen(str)-1]=='"'
              )
            { strcpy(os_std_format,str+strlen(ERRFOR));
              os_std_format[strlen(os_std_format)-1]='\0';
              strcpy(formatname,DEFAULTF);
              printf("format %s\n",os_std_format);
            }
         }
      }
  /*}}}  */
  if (strcmp(formatname,NO_FORM))
   /*{{{  print line and filename to stdout*/
   { F_L *s=f_list;
     char *f;
     char fn[1024];
     int l;
     int x;

     do
      { f=s->form; }
     while (*(s->name) && strcmp((s++)->name,formatname));
     str[0]='\0';
     if (in) fgets(str,4096,in);
     if (x=sscanf(str,f,fn,&l)!=2)
      { printf("%d\n",0); }
     else
      { if (*dir && *fn!=PATH_HOME)
           printf("%d\n%s%c%s\n",l,dir,PATH_SEP,fn);
        else
           printf("%d\n%s\n",l,fn);
      }
   }
   /*}}}  */
  else
   /*{{{  print the block to stdout*/
   {
     /*{{{  print the missing lines for the block*/
     while (blocklength--)
      { if (!eof_met)
         { str[0]='\0';
           eof_met= !in || (fgets(str,4096,in)==0);
         }
        if (eof_met)
           strcpy(str,"\n");
        if (lead) fputc(' ',stdout);
        fputs(str,stdout);
      }
     /*}}}  */
     /*{{{  print the header*/
     if (*title)
      { printf(title,blockbegin,startline,in?titlefile:titlestr);
        fputc('\n',stdout);
      }
     /*}}}  */
   }
   /*}}}  */
  /*{{{  close the file*/
  if (!std && in) fclose(in);
  /*}}}  */

  exit(0);
}
/*}}}  */
